/*++

	Copyright (c) 2009-2010 SMSC. All rights reserved.

	The information contained in this file is confidential and proprietary to
	Standard Microsystems Corporation.	No part of this file may be reproduced
	or distributed, in any form or by any means for any purpose, without the
	express written permission of Standard Microsystems Corporation.

Module Name:

	lan7500.h

Abstract:
	This module contains structure definitons and registers of LAN7500.

Revision History:
	01/19/2010	Initial version.

Notes:

--*/


#ifndef _LAN7500_H
#define _LAN7500_H

#include <ndis.h> 
#include "nuiouser.h"
#include "version.h" 
#include "os.h"
#include "Ioctl.h"

#define ON TRUE
#define OFF FALSE

#define LAN7500_TAG ((ULONG) '0057')	  //'0059' for the Tag '7500'

// media type, we use ethernet, change if necessary
#define NIC_MEDIA_TYPE					  NdisMedium802_3

// we use Internal, change to Pci, Isa, etc. properly
#define NIC_INTERFACE_TYPE				  NdisInterfaceInternal  

#define NDIS_ETH_TYPE_802_1Q				(0x8100)

// Timing values
#define NDIS_SLEEP_1_MS					1000
#define NDIS_SLEEP_5_MS					5000
#define NDIS_SLEEP_10_MS				10000
#define NDIS_SLEEP_1_S					1000000
#define NDIS_SLEEP_10_S					10000000

#define NIC_2_MIN_TIMEOUT				120000		// 2 mins in ms
#define NIC_10_SEC_TIMEOUT				10000		// 10 secs in ms
#define NIC_5_SEC_TIMEOUT				5000		// 5 secs in ms
#define NIC_2_SEC_TIMEOUT				2000		// 2 secs in ms
#define NIC_1_SEC_TIMEOUT				1000		// 1 secs in ms
#define NIC_HALF_SEC_TIMEOUT			500			// 0.5 secs in ms

// Highest byte is the NIC byte plus three vendor bytes, they are normally	
// obtained from the NIC. SMSC OUI = 00-80-0F, Thera NIC ID = 95
#define DEFAULT_NIC_VENDOR_ID				0x00800f					
#define DEFAULT_NIC_DEVICE_ID				0x75				   

typedef enum {
	NIC_STATUS_SUCCESS = 0,
	NIC_STATUS_FAILURE = 1
} NIC_STATUS, *PNIC_STATUS;

/* Definitions for requested media type*/
#define MEDIA_TYPE_AUTO_FULL                0x00
#define MEDIA_TYPE_10HD_AUTO                0x01
#define MEDIA_TYPE_10FD_AUTO                0x02
#define MEDIA_TYPE_100HD_AUTO               0x03
#define MEDIA_TYPE_100FD_AUTO               0x04
#define MEDIA_TYPE_1000FD_AUTO              0x05
#define MEDIA_TYPE_10HD_FORCED              0x06
#define MEDIA_TYPE_10FD_FORCED              0x07
#define MEDIA_TYPE_100HD_FORCED             0x08
#define MEDIA_TYPE_100FD_FORCED             0x09
#define MEDIA_TYPE_1000FD_MASTER_FORCED     0x0A
#define MEDIA_TYPE_1000FD_SLAVE_FORCED      0x0B
/*
// Receive Mask definitions 
#define ACCEPT_MULTICAST				0x0001
#define ACCEPT_BROADCAST				0x0002
#define PROMISCUOUS_MODE				0x0004
#define ACCEPT_SOURCE_ROUTING			0x0008
#define ACCEPT_ERR_PACKETS				0x0010
#define ACCEPT_ATT_MAC_FRAMES			0x0020
#define ACCEPT_MULTI_PROM				0x0040
#define TRANSMIT_ONLY					0x0080
#define ACCEPT_EXT_MAC_FRAMES			0x0100
#define EARLY_RX_ENABLE 				0x0200
#define PKT_SIZE_NOT_NEEDED 			0x0400
#define ACCEPT_SOURCE_ROUTING_SPANNING	0x0808
#define ACCEPT_ALL_MAC_FRAMES			0x0120
#define REJECT_DIRECTED 				0x1000
*/

/* Definitions for link speed */
#define LINK_SPEED_UNKNOWN                  0x0000
#define LINK_SPEED_10MBS                    0x0001
#define LINK_SPEED_100MBS                   0x0002
#define LINK_SPEED_1000MBS_MASTER           0x0004
#define LINK_SPEED_1000MBS_SLAVE            0x0008

#define LINK_DUPLEX_UNKNOWN         0x0000
#define LINK_DUPLEX_HALF            0x0001
#define LINK_DUPLEX_FULL            0x0002

/* Definitions for Auto MDIX */
#define AMDIX_DISABLE_STRAIGHT          0x00000000UL
#define AMDIX_DISABLE_CROSSOVER         0x00000001UL
#define AMDIX_ENABLE                    0x00000002UL
#define AMDIX_USE_STRAP                 0x00000003UL

// Tx MAT Cmds
#define     TX_CMD_A_LSO_               (0x08000000UL)      // Large Send Offload Enable
#define     TX_CMD_A_IPE_               (0x04000000UL)      // IP Checksum Offload Enable
#define     TX_CMD_A_TPE_               (0x02000000UL)      // TCP/UDP Checksum Offload Enable
#define     TX_CMD_A_IVTG_              (0x01000000UL)      // Insert VLAN Tag
#define     TX_CMD_A_RVTG_              (0x00800000UL)      // Replace VLAN Tag
#define     TX_CMD_A_FCS_               (0x00400000UL)      // Insert FCS and Pad
#define     TX_CMD_A_LEN_               (0x000FFFFFUL)      // Frame Length [19:0]


#define     TX_CMD_B_MSS_               (0x3FFF0000UL)      // Maximum Segment Size
#define     TX_CMD_B_VTAG_              (0x0000FFFFUL)      // VLAN Tag

// Rx sts Wd
#define     RX_CMD_A_ICE_               (0x80000000UL)      // IP Checksum Error
#define     RX_CMD_A_TCE_               (0x40000000UL)      // TCP/UDP Checksum Error
    #define     RX_CMD_A_CSE_               (0xC0000000UL)      // IP/TCP/UDP Checksum Error Mask
#define     RX_CMD_A_IPV_               (0x20000000UL)      // IP Version
#define     RX_CMD_A_PID_               (0x18000000UL)      // Protocol IP
    #define     RX_CMD_A_PID_NONE_IP_       (0x00000000UL) 
    #define     RX_CMD_A_PID_TCP_IP_        (0x08000000UL) 
    #define     RX_CMD_A_PID_UDP_IP_        (0x10000000UL) 
    #define     RX_CMD_A_PID_IP_            (0x18000000UL) 
#define     RX_CMD_A_PFF_               (0x04000000UL)      // Perfect Filter Passed
#define     RX_CMD_A_BAM_               (0x02000000UL)      // Broadcast Frame
#define     RX_CMD_A_MAM_               (0x01000000UL)      // Multicast Frame
#define     RX_CMD_A_FVTG_              (0x00800000UL)      // Frame is VLAN tagged
#define     RX_CMD_A_RED_               (0x00400000UL)      // Receive Error Detected
    #define     RX_CMD_A_RX_ERRS_           (0xC03F0000UL)  // FCS, DRP, RXE, LONG, RUNT, RWT, ICE, TCE
#define     RX_CMD_A_RWT_               (0x00200000UL)      // Receive Watchdog Timer Expired
#define     RX_CMD_A_RUNT_              (0x00100000UL)      // Runt Frame
#define     RX_CMD_A_LONG_              (0x00080000UL)      // Frame Too Long
#define     RX_CMD_A_RXE_               (0x00040000UL)      // Receive Error
#define     RX_CMD_A_DRB_               (0x00020000UL)      // Alignment Error
#define     RX_CMD_A_FCS_               (0x00010000UL)      // FCS Error
#define     RX_CMD_A_UAM_               (0x00008000UL)      // Unicast Frame
#define     RX_CMD_A_ICSM_              (0x00004000UL)      // Ignore TCP/UDP Checksum
#define     RX_CMD_A_LEN_               (0x00003FFFUL)      // Frame Length

#define     RX_CMD_B_CSUM_              (0xFFFF0000UL)      // Raw L3 Checksum
#define     RX_CMD_B_VTAG_              (0x0000FFFFUL)      // VLAN Tag

/*
 ****************************************************************************
 *    SCSRs
 ****************************************************************************
 */
#define ETH_NUMBER_OF_REGS              (192UL)

#define ETH_ID_REV                      (0x00UL)
    #define ETH_ID_REV_CHIP_ID_MASK_        (0xffff0000UL)
    #define ETH_ID_REV_CHIP_REV_MASK_       (0x0000ffffUL)
    #define ETH_ID_REV_CHIP_ID_7500_            (0x7500UL)  // LAN7500
    #define ETH_ID_REV_CHIP_ID_7505_            (0x7505UL)  // LAN7505, no MEF
    #define GetChipIdFromID_REV(dwReg)          ((dwReg & ETH_ID_REV_CHIP_ID_MASK_) >> 16)
    #define GetChiprevFromID_REV(dwReg)         (dwReg & ETH_ID_REV_CHIP_REV_MASK_)

#define ETH_FPGA_REV                    (0x04UL)

#define ETH_BOND_CTL                    (0x08UL)
    #define ETH_BOND_CTL_MEF_CODE_          (0x00446373UL)

#define ETH_INT_STS                     (0x0CUL)
    #define ETH_INT_STS_RDFO_               (0x00400000UL)  // Rx Data FIFO Overflow
    #define ETH_INT_STS_TXE_                (0x00200000UL)  // Transmitter Error
    #define ETH_INT_STS_MACRTO_             (0x00100000UL)  // Mac Reset Timeout
    #define ETH_INT_STS_TX_DIS_             (0x00080000UL)  // Tx Disabled
    #define ETH_INT_STS_RX_DIS_             (0x00040000UL)  // Rx Disabled
    #define ETH_INT_STS_PHY_INT_            (0x00020000UL)  // PHY Interrupt

    #define ETH_INT_STS_MAC_ERR_            (0x00008000UL)  // MAC Error Interrupt
    #define ETH_INT_STS_TDFU_               (0x00004000UL)  // Tx Data FIFO Underrun
    #define ETH_INT_STS_TDFO_               (0x00002000UL)  // Tx Data FIFO Overrun

    #define ETH_INT_STS_GPIO_               (0x00000FFFUL)  // GPIO Interrupt
        #define ETH_INT_STS_GPIO11_             (0x00000800UL)  // GPIO11
        #define ETH_INT_STS_GPIO10_             (0x00000400UL)  // GPIO10
        #define ETH_INT_STS_GPIO9_              (0x00000200UL)  // GPIO9
        #define ETH_INT_STS_GPIO8_              (0x00000100UL)  // GPIO8
        #define ETH_INT_STS_GPIO7_              (0x00000080UL)  // GPIO7
        #define ETH_INT_STS_GPIO6_              (0x00000040UL)  // GPIO6
        #define ETH_INT_STS_GPIO5_              (0x00000020UL)  // GPIO5
        #define ETH_INT_STS_GPIO4_              (0x00000010UL)  // GPIO4
        #define ETH_INT_STS_GPIO3_              (0x00000008UL)  // GPIO3
        #define ETH_INT_STS_GPIO2_              (0x00000004UL)  // GPIO2
        #define ETH_INT_STS_GPIO1_              (0x00000002UL)  // GPIO1
        #define ETH_INT_STS_GPIO0_              (0x00000001UL)  // GPIO0

#define ETH_HW_CFG                      (0x010UL)
    #define ETH_HW_CFG_SMDET_STS_           (0x00008000UL)  // Smart Detach Status
    #define ETH_HW_CFG_SMDET_EN_            (0x00004000UL)  // Smart Detach Enable
    #define ETH_HW_CFG_EEM_                 (0x00002000UL)  // EEPROM Emulation Enable
    #define ETH_HW_CFG_RST_PROTECT_         (0x00001000UL)  // Reset Protection
    #define ETH_HW_CFG_PORT_SWAP_           (0x00000800UL)  // Port Swap
    #define ETH_HW_CFG_PHY_BOOST_           (0x00000600UL)  // PHY Boost
        #define ETH_HW_CFG_PHY_BOOST_NORMAL_    (0x00000000UL)  // normal
        #define ETH_HW_CFG_PHY_BOOST_PLUS_4_    (0x00000200UL)  // +4%
        #define ETH_HW_CFG_PHY_BOOST_PLUS_8_    (0x00000400UL)  // +8%
        #define ETH_HW_CFG_PHY_BOOST_PLUS_12_   (0x00000600UL)  // +12%

    #define ETH_HW_CFG_LEDB_                (0x00000100UL)  // LED bypass, 1=15uS, 0=80 mS
    #define ETH_HW_CFG_BIR_                 (0x00000080UL)  // BULK IN response when emtpy:  1=NAK, 0=ZLP
    #define ETH_HW_CFG_SBP_                 (0x00000040UL)  // Stall BULK OUT Pipe Disable
    #define ETH_HW_CFG_IME_                 (0x00000020UL)  // Internal GMII Visibility Enable
    #define ETH_HW_CFG_MEF_                 (0x00000010UL)  // Multiple Ethernet Frames per USB Packet
    #define ETH_HW_CFG_ETC_                 (0x00000008UL)  // EEPROM Time-out Control
    #define ETH_HW_CFG_BCE_                 (0x00000004UL)  // Burst Cap Enable
    #define ETH_HW_CFG_LRST_                (0x00000002UL)  // Soft Lite Reset
    #define ETH_HW_CFG_SRST_                (0x00000001UL)  // Soft Reset

#define ETH_PMT_CTL                     (0x014UL)
    #define ETH_PMT_CTL_PHY_PWRUP_          (0x00000400UL)  // PHY Power-Up
    #define ETH_PMT_CTL_RES_CLR_WKP_MASK_   (0x00000300UL)
    #define ETH_PMT_CTL_RES_CLR_WKP_STS_    (0x00000200UL)  // Resume Clears Remote Wakeup Status
    #define ETH_PMT_CTL_RES_CLR_WKP_EN_     (0x00000100UL)  // Resume Clears Remote Wakeup Enable

    #define ETH_PMT_CTL_READY_                (0x00000080UL)  // Device Ready

    #define ETH_PMT_CTL_SUS_MODE_           (0x00000060UL)  // Suspend Mode
        #define ETH_PMT_CTL_SUS_MODE_0_         (0x00000000UL)  // SUSPEND0
        #define ETH_PMT_CTL_SUS_MODE_1_         (0x00000020UL)  // SUSPEND1
        #define ETH_PMT_CTL_SUS_MODE_2_         (0x00000040UL)  // SUSPEND2
        #define ETH_PMT_CTL_SUS_MODE_3_         (0x00000060UL)  // SUSPEND3

    #define ETH_PMT_CTL_PHY_RST_            (0x00000010UL)  //  PHY Reset
    #define ETH_PMT_CTL_WOL_EN_             (0x00000008UL)  //  Wake-On-Lan Enable
    #define ETH_PMT_CTL_ED_EN_              (0x00000004UL)  //  Energy Detect Enable
    #define ETH_PMT_CTL_WUPS_MASK_          (0x00000003UL)  //  Wakeup Status
        #define ETH_PMT_CTL_WUPS_MLT_           (0x00000003UL)  //  11=Multiple Wakeup asserted
        #define ETH_PMT_CTL_WUPS_MAC_           (0x00000002UL)  //  10=Wake-On-Lan
        #define ETH_PMT_CTL_WUPS_PHY_           (0x00000001UL)  //  01=Energy-Detect

#define ETH_LED_GPIO_CFG0               (0x018UL)
    #define ETH_LED_GPIO_CFG0_LED2_FUN_SEL_     (0x80000000UL)  // Set to Activity LED only
    #define ETH_LED_GPIO_CFG0_SPDLED_FUN_SEL_   (0x40000000UL)  // Set: LED0/LED1 are Link/Speed/Activity LEDs
																		// Clear: LED0/LED1 are Link/Speed LEDs
    #define ETH_LED_GPIO_CFG0_GPIOEN_        (0x0000F000UL)
        #define ETH_LED_GPIO_CFG0_GPIOEN0_       (0x00001000UL)
    #define ETH_LED_GPIO_CFG0_GPBUF_         (0x00000F00UL)
    #define ETH_LED_GPIO_CFG0_GPDIR_         (0x000000F0UL)
        #define ETH_LED_GPIO_CFG0_GPDIR0_        (0x00000010UL)
    #define ETH_LED_GPIO_CFG0_GPD_           (0x0000000FUL)

        
#define ETH_GPIO_CFG                    (0x01CUL)
    #define ETH_GPIO_CFG_GPEN_              (0xFF000000UL)
        #define ETH_GPIO_CFG_GPO4_EN_           (0x01000000UL)
    #define ETH_GPIO_CFG_GPTYPE             (0x00FF0000UL)
        #define ETH_GPIO_CFG_GPO4_TYPE          (0x00010000UL)
    #define ETH_GPIO_CFG_GPDIR_             (0x0000FF00UL)
        #define ETH_GPIO_CFG_GPO4_DIR_          (0x00000100UL)
    #define ETH_GPIO_CFG_GPDATA_            (0x000000FFUL)
        #define ETH_GPIO_CFG_GPO4_DATA_         (0x00000001UL)

#define ETH_GPIO_WAKE                   (0x020UL)
    #define ETH_GPIO_WAKE_GPIOPOL_          (0x0FFF0000UL)
    #define ETH_GPIO_WAKE_GPIOWK_           (0x00000FFFUL)

    #define ETH_GPIO_WAKE_PHY_LINKUP_EN_    (0x80000000UL)
    #define ETH_GPIO_WAKE_GPIOPOL7_         (0x00800000UL)
    #define ETH_GPIO_WAKE_GPIOPOL0_         (0x00010000UL)
    #define ETH_GPIO_WAKE_GPIOWK7_          (0x00000080UL)
    #define ETH_GPIO_WAKE_GPIOWK0_          (0x00000001UL)

#define ETH_DP_SEL                      (0x024UL)
    #define ETH_DP_SEL_DPRDY_               (0x80000000UL)  // Data Port Ready
    #define ETH_DP_SEL_RSEL_MASK_           (0x0000000FUL)  // RAM select
        #define ETH_DP_SEL_RSEL_WOL_RAM_        (0x00000006UL)    // Wol packet RAM
        #define ETH_DP_SEL_RSEL_DESC_RAM_       (0x00000005UL)
        #define ETH_DP_SEL_RSEL_TXFIFO_         (0x00000004UL)
        #define ETH_DP_SEL_RSEL_RXFIFO_         (0x00000003UL)
        #define ETH_DP_SEL_RSEL_LSO_            (0x00000002UL)
        #define ETH_DP_SEL_RSEL_VLAN_DA_        (0x00000001UL)    // RFE VLAN & DA [Hash]
        #define ETH_DP_SEL_RSEL_URXBUF_         (0x00000000UL)

#define ETH_DP_CMD              (0x028UL)
    #define ETH_DP_CMD_WRITE_               (0x00000001UL)
    #define ETH_DP_CMD_READ_                (0x00000000UL)

#define ETH_DP_ADDR             (0x02CUL)
    #define ETH_DP_ADDR_MASK_               (0x00003FFFUL)

#define ETH_DP_DATA             (0x030UL)

#define ETH_BURST_CAP           (0x034UL)

#define ETH_INT_EP_CTL          (0x038UL)
    #define ETH_INT_EP_INTEP_ON_            (0x80000000UL)  // Interrupt Endpoint Always On
    #define ETH_INT_EP_RDFO_EN_             (0x00400000UL)  // Rx Data FIFO Overflow
    #define ETH_INT_EP_TXE_EN_              (0x00200000UL)  // Transmitter Error
    #define ETH_INT_EP_MACRTO_EN_           (0x00100000UL)  // Mac Reset Timeout
    #define ETH_INT_EP_TX_DIS_EN_           (0x00080000UL)  // Tx Disabled
    #define ETH_INT_EP_RX_DIS_EN_           (0x00040000UL)  // Rx Disabled
    #define ETH_INT_EP_PHY_INT_EN_          (0x00020000UL)  // PHY Interrupt

    #define ETH_INT_EP_MAC_ERR_EN_          (0x00008000UL)  // MAC Error Interrupt
    #define ETH_INT_EP_TDFU_EN_             (0x00004000UL)  // Tx Data FIFO Underrun
    #define ETH_INT_EP_TDFO_EN_             (0x00002000UL)  // Tx Data FIFO Overrun
    #define ETH_INT_EP_RX_FIFO_EN_          (0x00001000UL)  // Rx FIFO Has Frame Enable ?!! 

    #define ETH_INT_EP_GPIO_                (0x00000FFFUL)  // GPIO Interrupt

#define ETH_BULK_IN_DLY         (0x03CUL)
    #define ETH_BULK_IN_DLY_AUTO            (0xFFFFFFFFUL)
    #define ETH_BULK_IN_DLY_DEFAULT_1_RCB   (0x00000000UL)
    #define ETH_BULK_IN_DLY_DEFAULT_N_RCBS  (0x00008000UL)

#define ETH_E2P_CMD             (0x040UL)
	
    #define ETH_E2P_CMD_EPC_BUSY_           (0x80000000UL)  // EPC Busy
    #define ETH_E2P_CMD_EPC_CMD_            (0x70000000UL)  // EPC Command
        #define ETH_E2P_CMD_EPC_CMD_RELOAD_     (0x70000000UL)
        #define ETH_E2P_CMD_EPC_CMD_ERAL_       (0x60000000UL)
        #define ETH_E2P_CMD_EPC_CMD_ERASE_      (0x50000000UL)
        #define ETH_E2P_CMD_EPC_CMD_WRAL_       (0x40000000UL)
        #define ETH_E2P_CMD_EPC_CMD_WRITE_      (0x30000000UL)
        #define ETH_E2P_CMD_EPC_CMD_EWEN_       (0x20000000UL)
        #define ETH_E2P_CMD_EPC_CMD_EWDS_       (0x10000000UL)
        #define ETH_E2P_CMD_EPC_CMD_READ_       (0x00000000UL)
	
    #define ETH_E2P_CMD_EPC_TIMEOUT_        (0x00000400UL)  // EPC Time-out
    #define ETH_E2P_CMD_E2P_PROG_DONE_      (0x00000200UL)  // EPC Data Loaded
    #define ETH_E2P_CMD_EPC_ADDR_           (0x000001FFUL)  // EPC Address
	
	
#define ETH_E2P_DATA            (0x044UL)
	
    #define ETH_E2P_DATA_EEPROM_DATA_       (0x000000FFUL)
	
#define ETH_STRAP_DBG           (0x048UL)
	
#define ETH_HS_ATTR        (0x04CUL)

#define ETH_FS_ATTR        (0x050UL)

#define ETH_STRNG_ATTR0        (0x54UL)

#define ETH_STRNG_ATTR1        (0x58UL)

#define ETH_FLAG_ATTR        (0x5CUL)

#define ETH_RFE_CTL             (0x060UL)
    #define ETH_RFE_CTL_TCP_UDP_COE_EN_     (0x00001000UL)  // Enable TCP/UDP Checksum Validation
    #define ETH_RFE_CTL_IP_COE_EN_          (0x00000800UL)  // Enable IP Checksum Validation
    #define ETH_RFE_CTL_BCAST_EN_           (0x00000400UL)  // Accept Broadcast Frames
    #define ETH_RFE_CTL_MCAST_EN_           (0x00000200UL)  // Accept Multicast Frames
    #define ETH_RFE_CTL_UCAST_EN_           (0x00000100UL)  // Accept All Unicast Frames
    #define ETH_RFE_CTL_VLAN_STRIP_         (0x00000080UL)  // Enable VLAN Tag Stripping
    #define ETH_RFE_CTL_DISCARD_UNTAGGED_   (0x00000040UL)  // Untag Frame Filtering
    #define ETH_RFE_CTL_VLAN_FILTER_        (0x00000020UL)  // Enable VLAN Filtering
    #define ETH_RFE_CTL_SA_FILTER_          (0x00000010UL)  // Enable Source Address Pefect Fitlering
    #define ETH_RFE_CTL_MCAST_HASH_         (0x00000008UL)  // Enable Multicast DA Only Hash Filtering
    #define ETH_RFE_CTL_DA_HASH_            (0x00000004UL)  // Enable Unicast DA Hash Filtering
    #define ETH_RFE_CTL_DA_PERFECT_         (0x00000002UL)  // Enable DA Pefect Filtering
    #define ETH_RFE_CTL_RST_                (0x00000001UL)  // Reset Receive Filtering Engine (NEVER SET IT)

#define ETH_VLAN_TYPE           (0x064UL)

#define ETH_FCT_RX_CTL          (0x090UL)
    #define ETH_FCT_RX_CTL_EN_              (0x80000000UL)  // FCT RX Enable
    #define ETH_FCT_RX_CTL_RST_             (0x40000000UL)  // FCT RX Reset
    #define ETH_FCT_RX_CTL_SBF_             (0x02000000UL)  // Store Bad Frames
    #define ETH_FCT_RX_CTL_OVFL_            (0x01000000UL)  // FCT RX Overflow
    #define ETH_FCT_RX_CTL_DROP_            (0x00800000UL)  // Frame Dropped
    #define ETH_FCT_RX_CTL_NOT_EMPTY_       (0x00400000UL)  // FCT RX Not Empty
    #define ETH_FCT_RX_CTL_EMPTY_           (0x00200000UL)  // FCT RX Empty
    #define ETH_FCT_RX_CTL_DIS_             (0x00100000UL)  // FCT RX Disabled
    #define ETH_FCT_RX_CTL_USED_            (0x0000FFFFUL)  // Data FIFO Used Space

#define ETH_FCT_TX_CTL          (0x094UL)
    #define ETH_FCT_TX_CTL_EN_              (0x80000000UL)  // FCT TX Enable
    #define ETH_FCT_TX_CTL_RST_             (0x40000000UL)  // FCT TX Reset
    #define ETH_FCT_TX_CTL_NOT_EMPTY_      (0x00400000UL)  // FCT TX Not Empty
    #define ETH_FCT_TX_CTL_EMPTY_           (0x00200000UL)  // FCT TX Empty
    #define ETH_FCT_TX_CTL_DIS_             (0x00100000UL)  // FCT TX Disabled
    #define ETH_FCT_TX_CTL_USED_            (0x0000FFFFUL)  // Data FIFO Used Space

#define ETH_FCT_RX_FIFO_END     (0x098UL)
    #define ETH_FCT_RX_FIFO_END_            (0x0000007FUL)

#define ETH_FCT_TX_FIFO_END     (0x09CUL)
    #define ETH_FCT_TX_FIFO_END_            (0x0000003FUL)

#define ETH_FCT_FLOW            (0x0A0UL)
    #define ETH_FCT_FLOW_OFF_               (0x00007F00UL)  // Flow Control Off Threshold
    #define ETH_FCT_FLOW_ON_                (0x0000007FUL)  // Flow Control On Threshold
 
#define ETH_MAC_CR              (0x100UL)
    #define ETH_MAC_CR_ADP_                 (0x00002000UL)  // Auto-Duplex Polarity
    #define ETH_MAC_CR_AUTO_DUPLEX_         (0x00001000UL)  // Auto-Duplex Detection
    #define ETH_MAC_CR_AUTO_SPEED_          (0x00000800UL)  // Auto-Speed Detection
    #define ETH_MAC_CR_LOOPBACK_            (0x00000400UL)  // Internal Loopback Operation Mode
    #define ETH_MAC_CR_BOLMT_               (0x000000C0UL)  // BackOff Limit
    #define ETH_MAC_CR_FULL_DUPLEX_         (0x00000008UL)  // Duplex Mode
    #define ETH_MAC_CR_SPEED_               (0x00000006UL)  // MAC Configuration
        #define ETH_MAC_CR_SPEED_1000_          (0x00000004UL)
        #define ETH_MAC_CR_SPEED_100_           (0x00000002UL) 
        #define ETH_MAC_CR_SPEED_10_            (0x00000000UL)
    #define ETH_MAC_CR_RST_                 (0x00000001UL)  // MAC Reset (NEVER SET THIS BIT)

#define ETH_MAC_RX              (0x104UL)
    #define ETH_MAC_RX_MAX_SIZE_SHIFT_      (16UL)
    #define ETH_MAC_RX_MAX_SIZE_            (0x3FFFUL<<ETH_MAC_RX_MAX_SIZE_SHIFT_)

    #define ETH_MAC_RX_FCS_STRIP_           (0x00000010UL)  // FCS Stripping
    #define ETH_MAC_RX_VLAN_FSE_            (0x00000004UL)  // VLAN Frame Size Enforcemnet
    #define ETH_MAC_RX_RXD_                 (0x00000002UL)  // Receiver Disabled
    #define ETH_MAC_RX_RXEN_                (0x00000001UL)  // Receiver Enable

#define ETH_MAC_TX              (0x108UL)
    #define ETH_MAC_TX_BAD_FCS_             (0x00000004UL)  // Bad FCS
    #define ETH_MAC_TX_TXD_                 (0x00000002UL)  // Transmitter Disabled
    #define ETH_MAC_TX_TXEN_                (0x00000001UL)  // Transmitter Enable

#define ETH_FLOW                (0x10CUL)
    #define ETH_FLOW_CR_FORCE_FC_           (0x80000000UL)  // Force Transmission of TX Flow Control Frame
    #define ETH_FLOW_CR_TX_FCEN_            (0x40000000UL)  // Tx Manual Flow Control Enable
    #define ETH_FLOW_CR_RX_FCEN_            (0x20000000UL)  // Rx Flow Control Enable
    #define ETH_FLOW_CR_FPF_                (0x10000000UL)  // Filter Pause Frames
    #define ETH_FLOW_CR_FCPT_               (0x0000FFFFUL)  // Pause Time

#define ETH_RAND_SEED           (0x110UL)

#define ETH_ERR_STS             (0x114UL)
    #define ETH_ERR_STS_FERR_               (0x00000100UL)  // FCS error (Rx)
    #define ETH_ERR_STS_LERR_               (0x00000080UL)  // Large (oversized) Rx frame
    #define ETH_ERR_STS_RFERR_              (0x00000040UL)  // Runt Frame received
    #define ETH_ERR_STS_ECERR_              (0x00000010UL)  // Excessive Collisions
    #define ETH_ERR_STS_ALERR_              (0x00000008UL)  // Rx Alignment error
    #define ETH_ERR_STS_URERR_              (0x00000004UL)  // TxUnderrun Error

#define ETH_RX_ADDRH            (0x118UL)

#define ETH_RX_ADDRL            (0x11CUL)

#define ETH_MII_ACC             (0x120UL)

    #define ETH_MII_ACC_PHY_ADDR_SHIFT_     (11UL)
    #define ETH_MII_ACC_PHY_ADDR_           (0x1FUL<<MII_ACC_PHY_ADDR_SHIFT_)

    #define ETH_MII_ACC_MIIRINDA_SHIFT_     (6UL)
    #define ETH_MII_ACC_MIIRINDA_           (0x1FUL<<MII_ACC_MIIRINDA_SHIFT_)

    #define ETH_MII_ACC_MII_WRITE_          (0x00000002UL)
    #define ETH_MII_ACC_MII_BUSY_           (0x00000001UL)


#define ETH_MII_DATA            (0x124UL)


#define ETH_WUCSR               (0x140UL)
    #define ETH_WUCSR_GUE_                  (0x00000200UL)  // Global Unicast Wakeup Enable

    #define ETH_WUCSR_PFDA_FR_              (0x00000080UL)  // Perfect DA Frame Received
    #define ETH_WUCSR_WUFR_                 (0x00000040UL)  // Remote Wakeup Frame Received
    #define ETH_WUCSR_MPR_                  (0x00000020UL)  // Magic Packet Received
    #define ETH_WUCSR_BCST_FR_              (0x00000010UL)  // Broadcast Frame Received

    #define ETH_WUCSR_PFDA_EN_              (0x00000008UL)  // Perfect DA Wakeup Enable
    #define ETH_WUCSR_WAKE_EN_              (0x00000004UL)  // Wakeup Frame Enable
    #define ETH_WUCSR_MPEN_                 (0x00000002UL)  // Magic Packet Enable
    #define ETH_WUCSR_BCST_EN_              (0x00000001UL)  // Broadcast Wakeup Enable

#define ETH_WUF_CFGx            (0x144UL)
    #define ETH_WUF_CFGx_EN_                (0x80000000UL)    // Filter Enable
    #define ETH_WUF_CFGx_TYPE_              (0x03000000UL)    // Filter Address Type
        #define ETH_WUF_CFGx_TYPE_MCAST_        (0x02000000UL)
        #define ETH_WUF_CFGx_TYPE_ALL_          (0x01000000UL)
        #define ETH_WUF_CFGx_TYPE_UCAST_        (0x00000000UL)

    #define ETH_WUF_CFGx_OFFSET_SHIFT_      (16UL)
    #define ETH_WUF_CFGx_OFFSET_            (0x7FUL<<WUF_CFGx_OFFSET_SHIFT_) 
    #define ETH_WUF_CFGx_CRC16_             (0x0000FFFFUL)

#define ETH_WUF_CFG0            (0x144UL)   // Wakeup Filter 0 Configuration Register
#define ETH_WUF_CFG1            (0x148UL)   // Wakeup Filter 1 Configuration Register
#define ETH_WUF_CFG2            (0x14CUL)   // Wakeup Filter 2 Configuration Register
#define ETH_WUF_CFG3            (0x150UL)   // Wakeup Filter 3 Configuration Register
#define ETH_WUF_CFG4            (0x154UL)   // Wakeup Filter 4 Configuration Register
#define ETH_WUF_CFG5            (0x158UL)   // Wakeup Filter 5 Configuration Register
#define ETH_WUF_CFG6            (0x15CUL)   // Wakeup Filter 6 Configuration Register
#define ETH_WUF_CFG7            (0x160UL)   // Wakeup Filter 7 Configuration Register

#define ETH_WUF_MASKx           (0x170UL)

#define ETH_WUF_MASK0_0         (0x170UL)   // Wakeup Filter 0 Byte Mask Registers
#define ETH_WUF_MASK0_1         (0x174UL)
#define ETH_WUF_MASK0_2         (0x178UL)
#define ETH_WUF_MASK0_3         (0x17CUL)

#define ETH_WUF_MASK1_0         (0x180UL)   // Wakeup Filter 1 Byte Mask Registers
#define ETH_WUF_MASK1_1         (0x184UL)
#define ETH_WUF_MASK1_2         (0x188UL)
#define ETH_WUF_MASK1_3         (0x18CUL)

#define ETH_WUF_MASK2_0         (0x190UL)   // Wakeup Filter 2 Byte Mask Registers
#define ETH_WUF_MASK2_1         (0x194UL)
#define ETH_WUF_MASK2_2         (0x198UL)
#define ETH_WUF_MASK2_3         (0x19CUL)

#define ETH_WUF_MASK3_0         (0x1A0UL)   // Wakeup Filter 3 Byte Mask Registers
#define ETH_WUF_MASK3_1         (0x1A4UL)
#define ETH_WUF_MASK3_2         (0x1A8UL)
#define ETH_WUF_MASK3_3         (0x1ACUL)

#define ETH_WUF_MASK4_0         (0x1B0UL)   // Wakeup Filter 4 Byte Mask Registers
#define ETH_WUF_MASK4_1         (0x1B4UL)
#define ETH_WUF_MASK4_2         (0x1B8UL)
#define ETH_WUF_MASK4_3         (0x1BCUL)

#define ETH_WUF_MASK5_0         (0x1C0UL)   // Wakeup Filter 5 Byte Mask Registers
#define ETH_WUF_MASK5_1         (0x1C4UL)
#define ETH_WUF_MASK5_2         (0x1C8UL)
#define ETH_WUF_MASK5_3         (0x1CCUL)

#define ETH_WUF_MASK6_0         (0x1D0UL)   // Wakeup Filter 6 Byte Mask Registers
#define ETH_WUF_MASK6_1         (0x1D4UL)
#define ETH_WUF_MASK6_2         (0x1D8UL)
#define ETH_WUF_MASK6_3         (0x1DCUL)

#define ETH_WUF_MASK7_0         (0x1E0UL)   // Wakeup Filter 7 Byte Mask Registers
#define ETH_WUF_MASK7_1         (0x1E4UL)
#define ETH_WUF_MASK7_2         (0x1E8UL)
#define ETH_WUF_MASK7_3         (0x1ECUL)

/*
 ****************************************************************************
 *  Perfect Address Filters
 ****************************************************************************
 */
 
#define ETH_MAF_BASE            (0x300UL)   // Base address of 32 DW pairs
#define ETH_MAF_SKIP            (0x08UL)    // skip bytes between each DW pair
#define ETH_MAF_HIx             (0x00UL)    // address offset from base of DW pair
#define ETH_MAF_LOx             (0x04UL)

#define ETH_MAF_HI_VALID_           (0x80000000UL)  // 1=entry is valid/enabled, 0=disabled
#define ETH_MAF_HI_TYPE_            (0x40000000UL)  // 1=SA, 0=DA (the normal case)

#define ETH_MAF_HI_0            (ETH_MAF_BASE + ETH_MAF_HIx)
#define ETH_MAF_LO_0            (ETH_MAF_BASE + ETH_MAF_LOx)
#define ETH_MAF_HI_1            (ETH_MAF_BASE + (1UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_1            (ETH_MAF_BASE + (1UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_2            (ETH_MAF_BASE + (2UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_2            (ETH_MAF_BASE + (2UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_3            (ETH_MAF_BASE + (3UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_3            (ETH_MAF_BASE + (3UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_4            (ETH_MAF_BASE + (4UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_4            (ETH_MAF_BASE + (4UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_5            (ETH_MAF_BASE + (5UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_5            (ETH_MAF_BASE + (5UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_6            (ETH_MAF_BASE + (6UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_6            (ETH_MAF_BASE + (6UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_7            (ETH_MAF_BASE + (7UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_7            (ETH_MAF_BASE + (7UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_8            (ETH_MAF_BASE + (8UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_8            (ETH_MAF_BASE + (8UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_9            (ETH_MAF_BASE + (9UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_9            (ETH_MAF_BASE + (9UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_10           (ETH_MAF_BASE + (10UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_10           (ETH_MAF_BASE + (10UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_11           (ETH_MAF_BASE + (11UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_11           (ETH_MAF_BASE + (11UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_12           (ETH_MAF_BASE + (12UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_12           (ETH_MAF_BASE + (12UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_13           (ETH_MAF_BASE + (13UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_13           (ETH_MAF_BASE + (13UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_14           (ETH_MAF_BASE + (14UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_14           (ETH_MAF_BASE + (14UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_15           (ETH_MAF_BASE + (15UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_15           (ETH_MAF_BASE + (15UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_16           (ETH_MAF_BASE + (16UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_16           (ETH_MAF_BASE + (16UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_17           (ETH_MAF_BASE + (17UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_17           (ETH_MAF_BASE + (17UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_18           (ETH_MAF_BASE + (18UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_18           (ETH_MAF_BASE + (18UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_19           (ETH_MAF_BASE + (19UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_19           (ETH_MAF_BASE + (19UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_20           (ETH_MAF_BASE + (20UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_20           (ETH_MAF_BASE + (20UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_21           (ETH_MAF_BASE + (21UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_21           (ETH_MAF_BASE + (21UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_22           (ETH_MAF_BASE + (22UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_22           (ETH_MAF_BASE + (22UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_23           (ETH_MAF_BASE + (23UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_23           (ETH_MAF_BASE + (23UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_24           (ETH_MAF_BASE + (24UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_24           (ETH_MAF_BASE + (24UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_25           (ETH_MAF_BASE + (25UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_25           (ETH_MAF_BASE + (25UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_26           (ETH_MAF_BASE + (26UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_26           (ETH_MAF_BASE + (26UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_27           (ETH_MAF_BASE + (27UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_27           (ETH_MAF_BASE + (27UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_28           (ETH_MAF_BASE + (28UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_28           (ETH_MAF_BASE + (28UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_29           (ETH_MAF_BASE + (29UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_29           (ETH_MAF_BASE + (29UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_30           (ETH_MAF_BASE + (30UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_30           (ETH_MAF_BASE + (30UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_31           (ETH_MAF_BASE + (31UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_31           (ETH_MAF_BASE + (31UL*ETH_MAF_SKIP) + ETH_MAF_LOx)
#define ETH_MAF_HI_32           (ETH_MAF_BASE + (32UL*ETH_MAF_SKIP) + ETH_MAF_HIx)
#define ETH_MAF_LO_32           (ETH_MAF_BASE + (32UL*ETH_MAF_SKIP) + ETH_MAF_LOx)

#define ETH_WUCSR2              (0x500UL)   // Wakeup Control and Status 2 Register
    #define ETH_WUCSR2_IPV4_TCPSYN_WAKE_EN_ (0x00000001UL)   // 
    #define ETH_WUCSR2_IPV6_TCPSYN_WAKE_EN_ (0x00000002UL)   // 
    #define ETH_WUCSR2_ARP_OFFLOAD_EN_      (0x00000004UL)   // 
    #define ETH_WUCSR2_NS_OFFLOAD_EN_       (0x00000008UL)   // 
    #define ETH_WUCSR2_IPV4_TCPSYN_RCD_     (0x00000010UL)   // 
    #define ETH_WUCSR2_IPV6_TCPSYN_RCD_     (0x00000020UL)   // 
    #define ETH_WUCSR2_ARP_RCD_             (0x00000040UL)   // 
    #define ETH_WUCSR2_NS_RCD_              (0x00000080UL)   // 

#define ETH_WOL_FIFO_STS        (0x504UL)   // 
    #define ETH_WOL_FIFO_STS_WOL_FIFO_OVFL_ (0x00000200UL)   // 
    #define ETH_WOL_FIFO_STS_WOL_FIFO_CNT_  (0x000001FFUL)   // 

#define ETH_NS1_IPV6_ADDR_DEST0 (0x510UL)   // 
#define ETH_NS1_IPV6_ADDR_DEST1 (0x514UL)   // 
#define ETH_NS1_IPV6_ADDR_DEST2 (0x518UL)   // 
#define ETH_NS1_IPV6_ADDR_DEST3 (0x51CUL)   // 

#define ETH_NS1_IPV6_ADDR_SRC0  (0x520UL)   // 
#define ETH_NS1_IPV6_ADDR_SRC1  (0x524UL)   // 
#define ETH_NS1_IPV6_ADDR_SRC2  (0x528UL)   // 
#define ETH_NS1_IPV6_ADDR_SRC3  (0x52CUL)   // 

#define ETH_NS1_ICMPV6_ADDR0_0  (0x530UL)   // 
#define ETH_NS1_ICMPV6_ADDR0_1  (0x534UL)   // 
#define ETH_NS1_ICMPV6_ADDR0_2  (0x538UL)   // 
#define ETH_NS1_ICMPV6_ADDR0_3  (0x53CUL)   // 

#define ETH_NS1_ICMPV6_ADDR1_0  (0x540UL)   // 
#define ETH_NS1_ICMPV6_ADDR1_1  (0x544UL)   // 
#define ETH_NS1_ICMPV6_ADDR1_2  (0x548UL)   // 
#define ETH_NS1_ICMPV6_ADDR1_3  (0x54CUL)   // 

#define ETH_NS2_IPV6_ADDR_DEST0 (0x550UL)   // 
#define ETH_NS2_IPV6_ADDR_DEST1 (0x554UL)   // 
#define ETH_NS2_IPV6_ADDR_DEST2 (0x558UL)   // 
#define ETH_NS2_IPV6_ADDR_DEST3 (0x55CUL)   // 

#define ETH_NS2_IPV6_ADDR_SRC0  (0x560UL)   // 
#define ETH_NS2_IPV6_ADDR_SRC1  (0x564UL)   // 
#define ETH_NS2_IPV6_ADDR_SRC2  (0x568UL)   // 
#define ETH_NS2_IPV6_ADDR_SRC3  (0x56CUL)   // 

#define ETH_NS2_ICMPV6_ADDR0_0  (0x570UL)   // 
#define ETH_NS2_ICMPV6_ADDR0_1  (0x574UL)   // 
#define ETH_NS2_ICMPV6_ADDR0_2  (0x578UL)   // 
#define ETH_NS2_ICMPV6_ADDR0_3  (0x57CUL)   // 

#define ETH_NS2_ICMPV6_ADDR1_0  (0x580UL)   // 
#define ETH_NS2_ICMPV6_ADDR1_1  (0x584UL)   // 
#define ETH_NS2_ICMPV6_ADDR1_2  (0x588UL)   // 
#define ETH_NS2_ICMPV6_ADDR1_3  (0x58CUL)   // 

#define ETH_SYN_IPV4_ADDR_SRC   (0x590UL)   // 

#define ETH_SYN_IPV4_ADDR_DEST  (0x594UL)   // 

#define ETH_SYN_IPV4_TCP_PORTS  (0x598UL)   // 
    #define ETH_SYN_IPV4_TCP_PORTS_IPV4_DEST_PORT_  (0xFFFF0000UL)   // 
    #define ETH_SYN_IPV4_TCP_PORTS_IPV4_SRC_PORT_   (0x0000FFFFUL)   // 

#define ETH_SYN_IPV6_ADDR_SRC0  (0x59CUL)   // 
#define ETH_SYN_IPV6_ADDR_SRC1  (0x5A0UL)   // 
#define ETH_SYN_IPV6_ADDR_SRC2  (0x5A4UL)   // 
#define ETH_SYN_IPV6_ADDR_SRC3  (0x5A8UL)   // 

#define ETH_SYN_IPV6_ADDR_DEST0 (0x5ACUL)   // 
#define ETH_SYN_IPV6_ADDR_DEST1 (0x5B0UL)   // 
#define ETH_SYN_IPV6_ADDR_DEST2 (0x5B4UL)   // 
#define ETH_SYN_IPV6_ADDR_DEST3 (0x5B8UL)   // 

#define ETH_SYN_IPV6_TCP_PORTS  (0x5BCUL)   // 
    #define ETH_SYN_IPV6_TCP_PORTS_IPV6_DEST_PORT_  (0xFFFF0000UL)   // 
    #define ETH_SYN_IPV6_TCP_PORTS_IPV6_SRC_PORT_   (0x0000FFFFUL)   // 

#define ETH_ARP_SPA             (0x5C0UL)   // 

#define ETH_ARP_TPA             (0x5C4UL)   // 

/*
 *  Hash Address Filters
 */
#define ETH_DA_HASH_BASE        (128UL)     // Base Addr of Destination Addr Hash Filter Table in RAM

#define ETH_WOL_FIFO_SIZE       (256UL)     

////////////////////////////////////
// PHY Definitions
////////////////////////////////////
#define LAN_PHY_ADDR        (1UL)

// Register Indexes
#define PHY_CTRL         0x00UL /* Control Register */
/* PHY Control Register */
    #define MII_CR_SPEED_SELECT_MSB_    (WORD)0x0040    /* bits 6,13: 10=1000, 01=100, 00=10 */
    #define MII_CR_COLL_TEST_ENABLE_    (WORD)0x0080    /* Collision test enable */
    #define MII_CR_FULL_DUPLEX_         (WORD)0x0100    /* FDX =1, half duplex =0 */
    #define MII_CR_RESTART_AUTO_NEG_    (WORD)0x0200    /* Restart auto negotiation */
    #define MII_CR_ISOLATE_             (WORD)0x0400    /* Isolate PHY from MII */
    #define MII_CR_POWER_DOWN_          (WORD)0x0800    /* Power down */
    #define MII_CR_AUTO_NEG_EN_         (WORD)0x1000    /* Auto Neg Enable */
    #define MII_CR_SPEED_SELECT_LSB_    (WORD)0x2000    /* bits 6,13: 10=1000, 01=100, 00=10 */
    #define MII_CR_LOOPBACK_            (WORD)0x4000    /* 0 = normal, 1 = loopback */
    #define MII_CR_RESET_               (WORD)0x8000    /* 0 = normal, 1 = PHY reset */

#define PHY_STATUS       0x01UL /* Status Regiser */
    #define MII_SR_EXTENDED_CAPS_       (WORD)0x0001    /* Extended register capabilities */
    #define MII_SR_JABBER_DETECT_       (WORD)0x0002    /* Jabber Detected */
    #define MII_SR_LINK_STATUS_         (WORD)0x0004    /* Link Status 1 = link */
    #define MII_SR_AUTONEG_CAPS_        (WORD)0x0008    /* Auto Neg Capable */
    #define MII_SR_REMOTE_FAULT_        (WORD)0x0010    /* Remote Fault Detect */
    #define MII_SR_AUTONEG_COMPLETE_    (WORD)0x0020    /* Auto Neg Complete */
    #define MII_SR_PREAMBLE_SUPPRESS_   (WORD)0x0040    /* Preamble may be suppressed */
    #define MII_SR_EXTENDED_STATUS_     (WORD)0x0100    /* Ext. status info in Reg 0x0F */
    #define MII_SR_100T2_HD_CAPS_       (WORD)0x0200    /* 100T2 Half Duplex Capable */
    #define MII_SR_100T2_FD_CAPS_       (WORD)0x0400    /* 100T2 Full Duplex Capable */
    #define MII_SR_10T_HD_CAPS_         (WORD)0x0800    /* 10T   Half Duplex Capable */
    #define MII_SR_10T_FD_CAPS_         (WORD)0x1000    /* 10T   Full Duplex Capable */
    #define MII_SR_100X_HD_CAPS_        (WORD)0x2000    /* 100X  Half Duplex Capable */
    #define MII_SR_100X_FD_CAPS_        (WORD)0x4000    /* 100X  Full Duplex Capable */
    #define MII_SR_100T4_CAPS_          (WORD)0x8000    /* 100T4 Capable */

#define PHY_ID1          0x02UL /* Phy Id Reg (word 1) */
#define PHY_ID2          0x03UL /* Phy Id Reg (word 2) */

#define PHY_AUTONEG_ADV  0x04UL /* Autoneg Advertisement */
    #define NWAY_AR_SELECTOR_FIELD_     (WORD)0x001F    /* indicates IEEE 802.3 CSMA/CD */
    #define NWAY_AR_10T_HD_CAPS_        (WORD)0x0020    /* 10T   Half Duplex Capable */
    #define NWAY_AR_10T_FD_CAPS_        (WORD)0x0040    /* 10T   Full Duplex Capable */
    #define NWAY_AR_100TX_HD_CAPS_      (WORD)0x0080    /* 100TX Half Duplex Capable */
    #define NWAY_AR_100TX_FD_CAPS_      (WORD)0x0100    /* 100TX Full Duplex Capable */
    #define NWAY_AR_100T4_CAPS_         (WORD)0x0200    /* 100T4 Capable */
    #define NWAY_AR_PAUSE_              (WORD)0x0400    /* Pause operation desired */
    #define NWAY_AR_ASM_DIR_            (WORD)0x0800    /* Asymmetric Pause Direction bit */
    #define NWAY_AR_REMOTE_FAULT_       (WORD)0x2000    /* Remote Fault detected */
    #define NWAY_AR_NEXT_PAGE_          (WORD)0x8000    /* Next Page ability supported */

    #define NWAY_AR_ALL_CAPS_           (NWAY_AR_10T_HD_CAPS_ | NWAY_AR_10T_FD_CAPS_ | \
                                         NWAY_AR_100TX_HD_CAPS_ | NWAY_AR_100TX_FD_CAPS_)
    #define NWAY_AR_PAUSE_MASK          (NWAY_AR_PAUSE_ | NWAY_AR_ASM_DIR_)
    
#define PHY_LP_ABILITY   0x05UL /* Link Partner Ability (Base Page) */
    #define NWAY_LPAR_SELECTOR_FIELD_   (WORD)0x001F    /* LP protocol selector field */
    #define NWAY_LPAR_10T_HD_CAPS_      (WORD)0x0020    /* LP is 10T   Half Duplex Capable */
    #define NWAY_LPAR_10T_FD_CAPS_      (WORD)0x0040    /* LP is 10T   Full Duplex Capable */
    #define NWAY_LPAR_100TX_HD_CAPS_    (WORD)0x0080    /* LP is 100TX Half Duplex Capable */
    #define NWAY_LPAR_100TX_FD_CAPS_    (WORD)0x0100    /* LP is 100TX Full Duplex Capable */
    #define NWAY_LPAR_100T4_CAPS_       (WORD)0x0200    /* LP is 100T4 Capable */
    #define NWAY_LPAR_PAUSE_            (WORD)0x0400    /* LP Pause operation desired */
    #define NWAY_LPAR_ASM_DIR_          (WORD)0x0800    /* LP Asymmetric Pause Direction bit */
    #define NWAY_LPAR_REMOTE_FAULT_     (WORD)0x2000    /* LP has detected Remote Fault */
    #define NWAY_LPAR_ACKNOWLEDGE_      (WORD)0x4000    /* LP has rx'd link code word */
    #define NWAY_LPAR_NEXT_PAGE_        (WORD)0x8000    /* Next Page ability supported */

#define PHY_AUTONEG_EXP  0x06UL /* Autoneg Expansion Reg */
    #define NWAY_ER_LP_NWAY_CAPS_       (WORD)0x0001    /* LP has Auto Neg Capability */
    #define NWAY_ER_PAGE_RXD_           (WORD)0x0002    /* Recv'd a new Page from LP  */
    #define NWAY_ER_NEXT_PAGE_CAPS_     (WORD)0x0004    /* We're NextPage capable (always 1 for this PHY) */
    #define NWAY_ER_LP_NEXT_PAGE_CAPS_  (WORD)0x0008    /* LP is NextPage capable */
    #define NWAY_ER_PAR_DETECT_FAULT_   (WORD)0x0010    /* Parallel Detection Fault */

#define PHY_NEXT_PAGE_TX 0x07UL /* Next Page TX */
    #define NPTX_MSG_CODE_FIELD_        (WORD)0x0001    /* NP msg code or unformatted data */
    #define NPTX_TOGGLE_                (WORD)0x0800    /* Toggles between exchanges of different NP */
    #define NPTX_ACKNOWLDGE2_           (WORD)0x1000    /* 1 = will comply with msg
                                                         * 0 = cannot comply with msg */
    #define NPTX_MSG_PAGE_              (WORD)0x2000    /* formatted(1)/unformatted(0) pg */
    #define NPTX_NEXT_PAGE_             (WORD)0x8000    /* 1 = addition NP will follow
                                                         * 0 = sending last NP */

#define PHY_LP_NEXT_PAGE 0x08UL /* Link Partner Next Page */
    #define LP_RNPR_MSG_CODE_FIELD_     (WORD)0x0001    /* NP msg code or unformatted data */
    #define LP_RNPR_TOGGLE_             (WORD)0x0800    /* Toggles between exchanges of different NP */
    #define LP_RNPR_ACKNOWLDGE2_        (WORD)0x1000    /* 1 = will comply with msg
                                                         * 0 = cannot comply with msg */
    #define LP_RNPR_MSG_PAGE_           (WORD)0x2000    /* formatted(1)/unformatted(0) pg */
    #define LP_RNPR_ACKNOWLDGE_         (WORD)0x4000    /* 1 = ACK / 0 = NO ACK */
    #define LP_RNPR_NEXT_PAGE_          (WORD)0x8000    /* 1 = addition NP will follow
                                                         * 0 = sending last NP */

#define PHY_1000T_CTRL   0x09UL /* 1000Base-T Control Reg */
    #define CR_1000T_ASYM_PAUSE_        (WORD)0x0080    /* Advertise asymmetric pause bit */
    #define CR_1000T_HD_CAPS_           (WORD)0x0100    /* Advertise 1000T HD capability */
    #define CR_1000T_FD_CAPS_           (WORD)0x0200    /* Advertise 1000T FD capability  */
    #define CR_1000T_REPEATER_DTE_      (WORD)0x0400    /* 1=Repeater/switch device port */
                                                        /* 0=DTE device */
    #define CR_1000T_MS_VALUE_          (WORD)0x0800    /* 1=Configure PHY as Master */
                                                        /* 0=Configure PHY as Slave */
    #define CR_1000T_MS_ENABLE_         (WORD)0x1000    /* 1=Master/Slave manual config value */
                                                        /* 0=Automatic Master/Slave config */
    #define CR_1000T_TEST_MODE_NORMAL_  (WORD)0x0000    /* Normal Operation */
    #define CR_1000T_TEST_MODE_1_       (WORD)0x2000    /* Transmit Waveform test */
    #define CR_1000T_TEST_MODE_2_       (WORD)0x4000    /* Master Transmit Jitter test */
    #define CR_1000T_TEST_MODE_3_       (WORD)0x6000    /* Slave Transmit Jitter test */
    #define CR_1000T_TEST_MODE_4_       (WORD)0x8000    /* Transmitter Distortion test */

#define PHY_1000T_STATUS 0x0AUL /* 1000Base-T Status Reg */
    #define SR_1000T_IDLE_ERROR_CNT_    (WORD)0x00FF    /* Num idle errors since last read */
    #define SR_1000T_ASYM_PAUSE_DIR_    (WORD)0x0100    /* LP asymmetric pause direction bit */
    #define SR_1000T_LP_HD_CAPS_        (WORD)0x0400    /* LP is 1000T HD capable */
    #define SR_1000T_LP_FD_CAPS_        (WORD)0x0800    /* LP is 1000T FD capable */
    #define SR_1000T_REMOTE_RX_STATUS_  (WORD)0x1000    /* Remote receiver OK */
    #define SR_1000T_LOCAL_RX_STATUS_   (WORD)0x2000    /* Local receiver OK */
    #define SR_1000T_MS_CONFIG_RES_     (WORD)0x4000    /* 1=Local TX is Master, 0=Slave */
    #define SR_1000T_MS_CONFIG_FAULT_   (WORD)0x8000    /* Master/Slave config fault */
    #define SR_1000T_REMOTE_RX_STATUS_SHIFT          12
    #define SR_1000T_LOCAL_RX_STATUS_SHIFT           13
    #define SR_1000T_PHY_EXCESSIVE_IDLE_ERR_COUNT    5
    #define FFE_IDLE_ERR_COUNT_TIMEOUT_20            20
    #define FFE_IDLE_ERR_COUNT_TIMEOUT_100           100

// Reserved Reg's
#define PHY_REG11       0x0BUL
#define PHY_REG12       0x0CUL
#define PHY_REG13       0x0DUL
#define PHY_REG14       0x0EUL

#define PHY_EXT_STATUS  0x0FUL /* Extended Status Reg */
    #define IEEE_ESR_1000T_HD_CAPS_     (WORD)0x1000    /* 1000T HD capable */
    #define IEEE_ESR_1000T_FD_CAPS_     (WORD)0x2000    /* 1000T FD capable */
    #define IEEE_ESR_1000X_HD_CAPS_     (WORD)0x4000    /* 1000X HD capable */
    #define IEEE_ESR_1000X_FD_CAPS_     (WORD)0x8000    /* 1000X FD capable */

    #define PHY_TX_POLARITY_MASK_       (WORD)0x0100    /* register 10h bit 8 (polarity bit) */
    #define PHY_TX_NORMAL_POLARITY_     (WORD)0         /* register 10h bit 8 (normal polarity) */

    #define AUTO_POLARITY_DISABLE_      (WORD)0x0010    /* register 11h bit 4 */
                                                        /* (0=enable, 1=disable) */

#define PHY_LINK_CTL   0x10UL /* Link Control Reg */
    #define SPEED_OPTIMIZE_CTRL_        (WORD)0x0300    /* Speed Optimize Control */
        #define SPEED_OPTIMIZE_CTRL_3_ATTEMPTS_    (WORD)0x0300
        #define SPEED_OPTIMIZE_CTRL_4_ATTEMPTS_    (WORD)0x0200
        #define SPEED_OPTIMIZE_CTRL_5_ATTEMPTS_    (WORD)0x0100
        #define SPEED_OPTIMIZE_CTRL_7_ATTEMPTS_    (WORD)0x0000
    #define BIST_START_                 (WORD)0x00C0    
        #define BIST_START_NORMAL_OPERATION_       (WORD)0x0000
        #define BIST_START_DIGITAL_BIST_ONLY_      (WORD)0x0040
    #define LINK_BRAKE_THRESHOLD_       (WORD)0x0030    
        #define LINK_BRAKE_THRESHOLD_10E_MINUS_8_  (WORD)0x0000
        #define LINK_BRAKE_THRESHOLD_10E_MINUS_9_  (WORD)0x0010
        #define LINK_BRAKE_THRESHOLD_10E_MINUS_10_ (WORD)0x0020
        #define LINK_BRAKE_THRESHOLD_10E_MINUS_11_ (WORD)0x0030
    #define LINK_BRAKE_ENABLE_          (WORD)0x0008    
    #define POWER_OPTIMIZE_DISABLE_     (WORD)0x0004    
    #define ENABLE_SPEED_SELECT_1000_   (WORD)0x0002    
    #define LOGIC_RESET_                (WORD)0x0001    

#define PHY_MOD_CTL_STS  0x11UL /* 10/100 Mode Control/Status Register */
    #define PHY_MOD_CTL_STS_EDSHORT_    (WORD)0x8000
    #define PHY_MOD_CTL_STS_FASTRIP_    (WORD)0x4000
    #define PHY_MOD_CTL_STS_EDPWRDOWN_  (WORD)0x2000
    #define PHY_MOD_CTL_STS_EDMODE_     (WORD)0x1000
    #define PHY_MOD_CTL_STS_MDPREBP_    (WORD)0x0400
    #define PHY_MOD_CTL_STS_FARLPBK_    (WORD)0x0200
    #define PHY_MOD_CTL_STS_FASTEST_    (WORD)0x0100
    #define PHY_MOD_CTL_STS_SPDOPT_EN_  (WORD)0x0080
    #define PHY_MOD_CTL_STS_AN_NP_EN_   (WORD)0x0040
    #define PHY_MOD_CTL_STS_AMDIX_DIS_  (WORD)0x0020
    #define PHY_MOD_CTL_STS_ANP_DIS_    (WORD)0x0010
    #define PHY_MOD_CTL_STS_PHYADBP_    (WORD)0x0008
    #define PHY_MOD_CTL_STS_FRC_GL_     (WORD)0x0004
    #define PHY_MOD_CTL_STS_ENERGYON_   (WORD)0x0002
    #define PHY_MOD_CTL_STS_XOVER_EN_   (WORD)0x0001

#define PHY_SPCL_MOD   0x12UL /* 10/100 Special Mode Register */
    #define PHY_SPCL_MOD_RXDV_EN_       (WORD)0x8000
    #define PHY_SPCL_MOD_10BT_LPBK_DIS_ (WORD)0x4000
    #define PHY_SPCL_MOD_CRC_ERR_SRC_   (WORD)0x0080
    #define PHY_SPCL_MOD_MCLK25EN_      (WORD)0x0040
    #define PHY_SPCL_MOD_PHYAD_         (WORD)0x000F

#define PHY_EXT_CTL_STS     0x13UL /* Extended Mode Control/Status Register */
    #define PHY_EXT_CTL_STS_MOD_        (WORD)0xF800
    #define PHY_EXT_CTL_STS_TXF_DEPTH_  (WORD)0x0600
    #define PHY_EXT_CTL_STS_MAC_INF_    (WORD)0x01C0
    #define PHY_EXT_CTL_STS_LED_MODE_   (WORD)0x0030
    #define PHY_EXT_CTL_STS_CLK125DIS_  (WORD)0x0008
    #define PHY_EXT_CTL_STS_MDIX_AB_    (WORD)0x0004
    #define PHY_EXT_CTL_STS_MDIX_CD_    (WORD)0x0002
    #define PHY_EXT_CTL_STS_PAR_DTCT_   (WORD)0x0001

#define PHY_SPCL_TST_CNTL   0x14UL /* Testability Control - Giga format*/
    #define PHY_SPCL_TST_CNTL_READ_     (WORD)0x8000
	#define PHY_SPCL_TST_CNTL_WRITE_     (WORD)0x4000
	#define PHY_SPCL_TST_CNTL_10_100_Mode_     (WORD)0x2000
	#define PHY_SPCL_TST_CNTL_REG_SET_     	(WORD)0x0380
    	#define PHY_SPCL_TST_CNTL_PCS_REG_     	(WORD)0x0000
		#define PHY_SPCL_TST_CNTL_DSP_REG_     	(WORD)0x0080
		#define PHY_SPCL_TST_CNTL_ANALOG_REG_   (WORD)0x0100
		#define PHY_SPCL_TST_CNTL_ADVANCE_REG_  (WORD)0x0180	
    #define PHY_SPCL_TST_CNTL_ADDR_ (WORD)0x007F
	

#define PHY_SPCL_TST_READ1   0x15UL /* Advanced Register Read Data Port */

#define PHY_SPCL_TST_READ2   0x16UL 

#define PHY_SPCL_TST_WRITE   0x17UL 



#define PHY_CTL_STS_IND     0x1BUL /* Control Status Indications */
    #define PHY_CTL_STS_IND_SQEOFF_     (WORD)0x0800
    #define PHY_CTL_STS_IND_XPOL_       (WORD)0x0010

#define PHY_CH_Q_STS        0x1CUL /* Channel Quality Status */
    #define PHY_CH_Q_STS_CBLN_          (WORD)0xF000
        #define PHY_CH_Q_STS_CBLN_0m_TO_10m_     (WORD)0x0000
        #define PHY_CH_Q_STS_CBLN_10m_TO_20m_    (WORD)0x1000
        #define PHY_CH_Q_STS_CBLN_20m_TO_30m_    (WORD)0x2000
        #define PHY_CH_Q_STS_CBLN_30m_TO_40m_	(WORD)0x3000
        #define PHY_CH_Q_STS_CBLN_40m_TO_50m_    (WORD)0x4000
        #define PHY_CH_Q_STS_CBLN_50m_TO_60m_    (WORD)0x5000
        #define PHY_CH_Q_STS_CBLN_60m_TO_70m_    (WORD)0x6000
        #define PHY_CH_Q_STS_CBLN_70m_TO_80m_    (WORD)0x7000
        #define PHY_CH_Q_STS_CBLN_80m_TO_90m_    (WORD)0x8000
        #define PHY_CH_Q_STS_CBLN_90m_TO_100m_   (WORD)0x9000
        #define PHY_CH_Q_STS_CBLN_1000m_TO_110m_ (WORD)0xA000
		
#define PHY_INT_SRC_FLG     0x1DUL /* Interrupt Source Flags */
		
#define PHY_INT_MASK        0x1EUL /* Interrupt Mask */
    #define PHY_INT_MASK_TX_BUF_OVF_    (WORD)0x0400
    #define PHY_INT_MASK_TX_BUF_UNF_    (WORD)0x0200
    #define PHY_INT_MASK_IDL_CNT_OVF_   (WORD)0x0100
    #define PHY_INT_MASK_ENERGY_ON_     (WORD)0x0080
    #define PHY_INT_MASK_AUTO_NEG_      (WORD)0x0040
    #define PHY_INT_MASK_RMT_FLT_DTCT_  (WORD)0x0020
    #define PHY_INT_MASK_LINK_DOWN_     (WORD)0x0010
    #define PHY_INT_MASK_MSTR_SLV_FLT_  (WORD)0x0004
    #define PHY_INT_MASK_AN_PAGE_RXD_   (WORD)0x0002
		
		
#define PHY_SPC_CTL_STS     0x1FUL /* PHY Special Control/Status Register */
    #define PHY_SPC_CTL_STS_AN_DONE_    (WORD)0x1000
    #define PHY_SPC_CTL_STS_SPD_IND_    (WORD)0x001C
        #define PHY_SPC_CTL_STS_SPD_IND_10HD_    (WORD)0x0004
        #define PHY_SPC_CTL_STS_SPD_IND_10FD_    (WORD)0x0014
        #define PHY_SPC_CTL_STS_SPD_IND_100HD_   (WORD)0x0008
        #define PHY_SPC_CTL_STS_SPD_IND_100FD_   (WORD)0x0018
    #define PHY_SPC_CTL_STS_SCR_DIS_    (WORD)0x0001
		
#define PHY_ANA_10BASE_T_HD         0x01
#define PHY_ANA_10BASE_T_FD         0x02
#define PHY_ANA_100BASE_TX_HD       0x04
#define PHY_ANA_100BASE_TX_FD       0x08
#define PHY_ANA_1000BASE_T_FD       0x10
#define PHY_ANA_ALL_SUPPORTED_MEDIA (PHY_ANA_10BASE_T_HD |   \
											 PHY_ANA_10BASE_T_FD |	 \
											 PHY_ANA_100BASE_TX_HD | \
											 PHY_ANA_100BASE_TX_FD | \
											 PHY_ANA_1000BASE_T_FD)

/******************************************************************************/
/* Bit Mask definitions 													  */
/******************************************************************************/
#define BIT_NONE					0x00000000
#define BIT_0						0x0001
#define BIT_1						0x0002
#define BIT_2						0x0004
#define BIT_3						0x0008
#define BIT_4						0x0010
#define BIT_5						0x0020
#define BIT_6						0x0040
#define BIT_7						0x0080
#define BIT_8						0x0100
#define BIT_9						0x0200
#define BIT_10						0x0400
#define BIT_11						0x0800
#define BIT_12						0x1000
#define BIT_13						0x2000
#define BIT_14						0x4000
#define BIT_15						0x8000
#define BIT_16						0x10000
#define BIT_17						0x20000
#define BIT_18						0x40000
#define BIT_19						0x80000
#define BIT_20						0x100000
#define BIT_21						0x200000
#define BIT_22						0x400000
#define BIT_23						0x800000
#define BIT_24						0x1000000
#define BIT_25						0x2000000
#define BIT_26						0x4000000
#define BIT_27						0x8000000
#define BIT_28						0x10000000
#define BIT_29						0x20000000
#define BIT_30						0x40000000
#define BIT_31						0x80000000

// Vendor Requests
#define     USB_VENDOR_REQUEST_WRITE_REGISTER   0xA0
#define     USB_VENDOR_REQUEST_READ_REGISTER    0xA1
#define     USB_VENDOR_REQUEST_GET_STATS        0xA2

// Interrupt Endpoint status word bitfields 
// -- fixme needs to be updated when info becomes available in the PAS
#define     INT_ENP_MACRTO_                     ETH_INT_STS_MACRTO_
#define     INT_ENP_TX_STOP_                    ETH_INT_STS_TX_DIS_
#define     INT_ENP_RX_STOP_                    ETH_INT_STS_RX_DIS_
#define     INT_ENP_PHY_INT_                    ETH_INT_STS_PHY_INT_
#define     INT_ENP_TXE_                        ETH_INT_STS_TXE_    
#define     INT_ENP_TDFU_                       ETH_INT_STS_TDFU_   
#define     INT_ENP_TDFO_                       ETH_INT_STS_TDFO_   
#define     INT_ENP_RFHF_                       (0x00001000UL)
#define     INT_ENP_GPIOS_                      ETH_INT_STS_GPIO_  

/******************************************************************************/
/* Wake on LAN pattern structure and definitions                              */
/******************************************************************************/
#define MAX_ARP_OFFLOAD_ADDRESSES   1
#define MAX_NS_OFFLOAD_ADDRESSES    2

#define MAX_WOL_PACKET_SAVE_BUFFER  256
#define MAX_OFFSET_OF_WOL_PATTERN   255
#define NUMBER_OF_WOL_PATTERNS_8    8       // Assumed <= 255
#define SIZE_OF_WOL_PATTERN         128
#define SIZE_OF_WOL_PATTERN_IN_DW   4
// --fixme : What if user enables jumbo packets? Does NDIS sends jumbo WOL patterns to the driver?
#define SIZE_OF_WOL_SAMPLE          2048    // Enough to hold any Ethernet frame.


// Stats block structures
typedef struct _LAN7500_RX_STATS {
	DWORD	RxGoodFrames;
	DWORD	RxCrcErrors;
	DWORD	RxRuntFrameErrors;
	DWORD	RxAlignmentErrors;
	DWORD	RxFrameTooLongError;
	DWORD	RxLaterCollisionError;
	DWORD	RxBadFrames;
	DWORD	RxFifoDroppedFrames;
}LAN7500_RX_STATS, *PLAN7500_RX_STATS;

typedef struct _LAN7500_TX_STATS {
	DWORD	TxGoodFrames;
	DWORD	TxPauseFrames;
	DWORD	TxSingleCollisions;
	DWORD	TxMultipleCollisions;
	DWORD	TxExcessiveCollisionErrors;
	DWORD	TxLateCollisionErrors;
	DWORD	TxBufferUnderrunErrors;
	DWORD	TxExcessiveDeferralErrors;
	DWORD	TxCarrierErrors;
	DWORD	TxBadFrames;
} LAN7500_TX_STATS, *PLAN7500_TX_STATS;


/******************************************************************************/
/* Wake on LAN pattern structure and definitions							  */
/******************************************************************************/

#define MAX_OFFSET_OF_WOL_PATTERN	255
#define NUMBER_OF_WOL_PATTERNS_4	4		// Assumed <= 255
#define NUMBER_OF_WOL_PATTERNS_8	8		// Assumed <= 255
#define SIZE_OF_WOL_PATTERN			128
#define SIZE_OF_WOL_PATTERN_IN_DW	4
#define SIZE_OF_WOL_SAMPLE			2048	// Enough to hold any Ethernet frame.

typedef struct{
	DWORD	Enabled;
	BYTE	HeadPattern;
	BYTE	*pWOLSample;
	DWORD	WOLSampleSize;
	BYTE	*pWOLMask;
	DWORD	WOLMaskSize;
	WORD	FirstEnabledByte;
	WORD	LastEnabledByte;
} LM_WOL_SAMPLE_STRUCT, *PLM_WOL_SAMPLE_STRUCT;

typedef UCHAR	NIC_MAC_ADDRESS[6];

#define 	ETH_ADDR_SIZE				6
#define 	ETH_HEADER_SIZE 			14
#define 	ETH_MAX_DATA_SIZE			1500
#define 	ETH_MAX_PACKET_SIZE 		(ETH_HEADER_SIZE + ETH_MAX_DATA_SIZE+4)
#define 	ETH_MIN_PACKET_SIZE 		60
#define 	ETH_MAX_FRAME_SIZE			(ETH_MAX_PACKET_SIZE + 8UL) 
#define 	NIC_MEDIA_MAX_SPEED 		1000000000     //bits per sec.

#define ETH_IS_LOCALLY_ADMINISTERED(Address) \
	(BOOLEAN)(((PUCHAR)(Address))[0] & ((UCHAR)0x02))

#define NIC_MAX_MCAST_LIST				512

#define NIC_MIN_VAL_FOR_MAX_BUSY_SENDS	1
#define NIC_MIN_VAL_FOR_MAX_BUSY_RECVS	1
#define NIC_MIN_VAL_FOR_MAX_BUSY_INTS	1	

#define NIC_MAX_BUSY_SENDS_DEFAULT		2
#define NIC_MAX_BUSY_RECVS_DEFAULT		2
#define NIC_MAX_BUSY_INTS_DEFAULT		2	

#define NIC_MIN_VAL_FOR_RECVS_LOW_WM	1
#define NIC_MIN_VAL_FOR_INTS_LOW_WM		1	

#define NIC_MAX_LOOKAHEAD				ETH_MAX_DATA_SIZE
//#define NIC_BUFFER_SIZE 				(2 * 1024)
#define INT_BUFFER_SIZE 				4UL	// (Int Status is 1 dword)
#define NIC_ADAPTER_NAME_SIZE			128

#define BUFFER_SIZE_4KB						(4 * 1024)
#define BUFFER_SIZE_6KB						(6 * 1024)
#define BUFFER_SIZE_8256_BYTES				(6 * 1024 + 33 * 64)
#define BUFFER_SIZE_16KB					(16 * 1024)
#define BUFFER_SIZE_18_5KB					(16 * 1024 + 5 * 512)

#define BUFFER_SIZE_8KB					(8 * 1024)
#define BUFFER_SIZE_10KB                    (10 * 1024)

//
// Buffer size passed in NdisMQueryAdapterResources 						   
// We should only need three adapter resources (IO, interrupt and memory),
// Some devices get extra resources, so have room for 10 resources 
//
#define NIC_RESOURCE_BUF_SIZE			(sizeof(NDIS_RESOURCE_LIST) + \
										(10*sizeof(CM_PARTIAL_RESOURCE_DESCRIPTOR)))

#define NIC_SUPPORTED_FILTERS ( \
				NDIS_PACKET_TYPE_DIRECTED	| \
				NDIS_PACKET_TYPE_MULTICAST	| \
				NDIS_PACKET_TYPE_BROADCAST	| \
				NDIS_PACKET_TYPE_PROMISCUOUS | \
				NDIS_PACKET_TYPE_ALL_MULTICAST)

#define fMP_INIT_IN_PROGRESS					0x00000001
#define fMP_RESET_IN_PROGRESS					0x00000002
#define fMP_DISCONNECTED						0x00000004
#define fMP_HALT_IN_PROGRESS					0x00000008
#define fMP_SURPRISE_REMOVED					0x00000010
#define fMP_SEND_SIDE_RESOURCE_ALLOCATED		0x00000020
#define fMP_RECV_SIDE_RESOURCE_ALLOCATED		0x00000040
#define fMP_INT_RESOURCE_ALLOCATED				0x00000080
#define fMP_POST_INTS							0x00000100
#define fMP_POST_READS							0x00001000
#define fMP_IGNORE_READS						0x00002000
#define fMP_POST_WRITES							0x00004000
#define fMP_PERIODIC_LINK_CHECK					0x00008000
#define fMP_LINK_CHECK_IN_PROGRESS				0x00010000
#define fMP_REQUEST_PENDING						0x00020000
#define fMP_USB_RESET_IN_PROGRESS				0x00040000
#define fMP_NEED_USB_RESET						0x00080000
#define fMP_NEED_FULL_RESET						0x00100000
#define fMP_IOCTL_REGISTERED					0x00200000
#define fMP_DRVCMD_IN_PROGRESS					0x00400000
#define	fMP_DEVICE_NOT_ATTACHED					0x00800000


typedef struct	_SHOW_REG	{
	WCHAR	wszName[50];
	USHORT	 dwOffset;
}	SHOW_REG;




static const SHOW_REG	sysCsr[] = {  
										{L"0x0 ETH_ID_REV 	 ", ETH_ID_REV},
                                        {L"0x4 ETH_FPGA_REV 	 ",ETH_FPGA_REV},
                                        {L"0x8 ETH_BOND_CTL 	 ",ETH_BOND_CTL},
                                        {L"0xc ETH_INT_STS 	 ",ETH_INT_STS},
                                        {L"0x10 ETH_HW_CFG 	 ",ETH_HW_CFG},
                                        {L"0x14 ETH_PMT_CTL 	 ",ETH_PMT_CTL},
                                        {L"0x18 ETH_LED_GPIO_CFG0 	 ",ETH_LED_GPIO_CFG0},
                                        {L"0x1c ETH_GPIO_CFG 	 ",ETH_GPIO_CFG},
                                        {L"0x20 ETH_GPIO_WAKE 	 ",ETH_GPIO_WAKE},
                                        {L"0x24 ETH_DP_SEL 	 ",ETH_DP_SEL},
                                        {L"0x28 ETH_DP_CMD 	 ",ETH_DP_CMD},
                                        {L"0x2c ETH_DP_ADDR 	 ",ETH_DP_ADDR},
                                        {L"0x30 ETH_DP_DATA	 ",ETH_DP_DATA},
                                        {L"0x34 ETH_BURST_CAP 	 ",ETH_BURST_CAP},
                                        {L"0x38 ETH_INT_EP_CTL 	 ",ETH_INT_EP_CTL},
                                        {L"0x3c ETH_BULK_IN_DLY 	 ",ETH_BULK_IN_DLY},
                                        {L"0x40 ETH_E2P_CMD 	 ",ETH_E2P_CMD},
                                        {L"0x44 ETH_E2P_DATA 	 ",ETH_E2P_DATA},
                                        {L"0x48 ETH_STRAP_DBG 	 ",ETH_STRAP_DBG},
                                        {L"0x04C ETH_HS_ATTR 	 ",ETH_HS_ATTR },  
										{L"0x050 ETH_FS_ATTR 	 ", ETH_FS_ATTR },     	
										{L"0x054 ETH_STRNG_ATTR0 	 ",ETH_STRNG_ATTR0 },       
										{L"0x058 ETH_STRNG_ATTR1 	 ", ETH_STRNG_ATTR1 },       
										{L"0x05C ETH_FLAG_ATTR 	 ", ETH_FLAG_ATTR  },      
                                        {L"0x60 ETH_RFE_CTL 	 ",ETH_RFE_CTL},
                                        {L"0x64 ETH_VLAN_TYPE 	 ",ETH_VLAN_TYPE},
                                        {L"0x90 ETH_FCT_RX_CTL 	 ",ETH_FCT_RX_CTL},
                                        {L"0x94 ETH_FCT_TX_CTL 	 ",ETH_FCT_TX_CTL},
                                        {L"0x98 ETH_FCT_RX_FIFO_END 	 ",ETH_FCT_RX_FIFO_END},
                                        {L"0x9c ETH_FCT_TX_FIFO_END 	 ",ETH_FCT_TX_FIFO_END},
                                        {L"0xa0 ETH_FCT_FLOW 	 ",ETH_FCT_FLOW},
                                        {L"0x100 ETH_MAC_CR 	 ",ETH_MAC_CR},
                                        {L"0x104 ETH_MAC_RX 	 ",ETH_MAC_RX},
                                        {L"0x108 ETH_MAC_TX 	 ",ETH_MAC_TX},
                                        {L"0x10c ETH_FLOW 	 ",ETH_FLOW},
                                        {L"0x110 ETH_RAND_SEED 	 ",ETH_RAND_SEED},
                                        {L"0x114 ETH_ERR_STS 	 ",ETH_ERR_STS},
                                        {L"0x118 ETH_RX_ADDRH 	 ",ETH_RX_ADDRH},
                                        {L"0x11c ETH_RX_ADDRL 	 ",ETH_RX_ADDRL},
                                        {L"0x120 ETH_MII_ACC 	 ",ETH_MII_ACC},
                                        {L"0x124 ETH_MII_DATA 	 ",ETH_MII_DATA},
                                        {L"0x140 ETH_WUCSR 	 ",ETH_WUCSR},
                                        /*
                                        {L"ETH_WUF_CFG0 	 ",ETH_WUF_CFG0},
                                        {L"ETH_WUF_CFG1 	 ",ETH_WUF_CFG1},
                                        {L"ETH_WUF_CFG2 	 ",ETH_WUF_CFG2},
                                        {L"ETH_WUF_CFG3 	 ",ETH_WUF_CFG3},
                                        {L"ETH_WUF_CFG4 	 ",ETH_WUF_CFG4},
                                        {L"ETH_WUF_CFG5 	 ",ETH_WUF_CFG5},
                                        {L"ETH_WUF_CFG6 	 ",ETH_WUF_CFG6},
                                        {L"ETH_WUF_CFG7 	 ",ETH_WUF_CFG7},
                                        					*/
										{L"0x170 ETH_WUF_MASK 	 ", ETH_WUF_MASKx},
						/*
                                        {L"ETH_WUF_MASK0_0 	 ",ETH_WUF_MASK0_0},
                                        {L"ETH_WUF_MASK0_1 	 ",ETH_WUF_MASK0_1},
                                        {L"ETH_WUF_MASK0_2 	 ",ETH_WUF_MASK0_2},
                                        {L"ETH_WUF_MASK0_3	 ",ETH_WUF_MASK0_3},
                                        {L"ETH_WUF_MASK1_0 	 ",ETH_WUF_MASK1_0},
                                        {L"ETH_WUF_MASK1_1 	 ",ETH_WUF_MASK1_1},
                                        {L"ETH_WUF_MASK1_2 	 ",ETH_WUF_MASK1_2},
                                        {L"ETH_WUF_MASK1_3 	 ",ETH_WUF_MASK1_3},
                                        {L"ETH_WUF_MASK2_0 	 ",ETH_WUF_MASK2_0},
                                        {L"ETH_WUF_MASK2_1 	 ",ETH_WUF_MASK2_1},
                                        {L"ETH_WUF_MASK2_2 	 ",ETH_WUF_MASK2_2},
                                        {L"ETH_WUF_MASK2_3 	 ",ETH_WUF_MASK2_3},
                                        {L"ETH_WUF_MASK3_0 	 ",ETH_WUF_MASK3_0},
                                        {L"ETH_WUF_MASK3_1 	 ",ETH_WUF_MASK3_1},
                                        {L"ETH_WUF_MASK3_2 	 ",ETH_WUF_MASK3_2},
                                        {L"ETH_WUF_MASK3_3 	 ",ETH_WUF_MASK3_3},
                                        {L"ETH_WUF_MASK4_0 	 ",ETH_WUF_MASK4_0},
                                        {L"ETH_WUF_MASK4_1 	 ",ETH_WUF_MASK4_1},
                                        {L"ETH_WUF_MASK4_2 	 ",ETH_WUF_MASK4_2},
                                        {L"ETH_WUF_MASK4_3 	 ",ETH_WUF_MASK4_3},
                                        {L"ETH_WUF_MASK5_0 	 ",ETH_WUF_MASK5_0},
                                        {L"ETH_WUF_MASK5_1 	 ",ETH_WUF_MASK5_1},
                                        {L"ETH_WUF_MASK5_2 	 ",ETH_WUF_MASK5_2},
                                        {L"ETH_WUF_MASK5_3 	 ",ETH_WUF_MASK5_3},
                                        {L"ETH_WUF_MASK6_0 	 ",ETH_WUF_MASK6_0},
                                        {L"ETH_WUF_MASK6_1 	 ",ETH_WUF_MASK6_1},
                                        {L"ETH_WUF_MASK6_2 	 ",ETH_WUF_MASK6_2},
                                        {L"ETH_WUF_MASK6_3 	 ",ETH_WUF_MASK6_3},
                                        {L"ETH_WUF_MASK7_0 	 ",ETH_WUF_MASK7_0},
                                        {L"ETH_WUF_MASK7_1 	 ",ETH_WUF_MASK7_1},
                                        {L"ETH_WUF_MASK7_2 	 ",ETH_WUF_MASK7_2},
                                        {L"ETH_WUF_MASK7_3	 ",ETH_WUF_MASK7_3},
                                        {L"ETH_MAF_HI_0 	 ",ETH_MAF_HI_0},
                                        {L"ETH_MAF_LO_0 	 ",ETH_MAF_LO_0},
                                        {L"ETH_MAF_HI_1 	 ",ETH_MAF_HI_1},
                                        {L"ETH_MAF_LO_1	     ",ETH_MAF_LO_1},
                                        {L"ETH_MAF_HI_2 	 ",ETH_MAF_HI_2},
                                        {L"ETH_MAF_LO_2 	 ",ETH_MAF_LO_2},
                                        {L"ETH_MAF_HI_3	     ",ETH_MAF_HI_3},
                                        {L"ETH_MAF_LO_3 	 ",ETH_MAF_LO_3},
                                        {L"ETH_MAF_HI_4 	 ",ETH_MAF_HI_4},
                                        {L"ETH_MAF_LO_4 	 ",ETH_MAF_LO_4},
                                        {L"ETH_MAF_HI_5 	 ",ETH_MAF_HI_5},
                                        {L"ETH_MAF_LO_5 	 ",ETH_MAF_LO_5},
                                        {L"ETH_MAF_HI_6 	 ",ETH_MAF_HI_6},
                                        {L"ETH_MAF_LO_6 	 ",ETH_MAF_LO_6},
                                        {L"ETH_MAF_HI_7 	 ",ETH_MAF_HI_7},
                                        {L"ETH_MAF_LO_7	     ",ETH_MAF_LO_7},
                                        {L"ETH_MAF_HI_8 	 ",ETH_MAF_HI_8},
                                        {L"ETH_MAF_LO_8 	 ",ETH_MAF_LO_8},
                                        {L"ETH_MAF_HI_9	     ",ETH_MAF_HI_9},
                                        {L"ETH_MAF_LO_9 	 ",ETH_MAF_LO_9},
                                        {L"ETH_MAF_HI_10	 ",ETH_MAF_HI_10},
                                        {L"ETH_MAF_LO_10 	 ",ETH_MAF_LO_10},
                                        {L"ETH_MAF_HI_11 	 ",ETH_MAF_HI_11},
                                        {L"ETH_MAF_LO_11 	 ",ETH_MAF_LO_11},
                                        {L"ETH_MAF_HI_12 	 ",ETH_MAF_HI_12},
                                        {L"ETH_MAF_LO_12 	 ",ETH_MAF_LO_12},
                                        {L"ETH_MAF_HI_13 	 ",ETH_MAF_HI_13},
                                        {L"ETH_MAF_LO_13 	 ",ETH_MAF_LO_13},
                                        {L"ETH_MAF_HI_14 	 ",ETH_MAF_HI_14},
                                        {L"ETH_MAF_LO_14 	 ",ETH_MAF_LO_14},
                                        {L"ETH_MAF_HI_15 	 ",ETH_MAF_HI_15},
                                        {L"ETH_MAF_LO_15 	 ",ETH_MAF_LO_15},
                                        {L"ETH_MAF_HI_16 	 ",ETH_MAF_HI_16},
                                        {L"ETH_MAF_LO_16 	 ",ETH_MAF_LO_16},
                                        {L"ETH_MAF_HI_17 	 ",ETH_MAF_HI_17},
                                        {L"ETH_MAF_LO_17 	 ",ETH_MAF_LO_17},
                                        {L"ETH_MAF_HI_18 	 ",ETH_MAF_HI_18},
                                        {L"ETH_MAF_LO_18 	 ",ETH_MAF_LO_18},
                                        {L"ETH_MAF_HI_19 	 ",ETH_MAF_HI_19},
                                        {L"ETH_MAF_LO_19 	 ",ETH_MAF_LO_19},
                                        {L"ETH_MAF_HI_20 	 ",ETH_MAF_HI_20},
                                        {L"ETH_MAF_LO_20 	 ",ETH_MAF_LO_20},
                                        {L"ETH_MAF_HI_21 	 ",ETH_MAF_HI_21},
                                        {L"ETH_MAF_LO_21 	 ",ETH_MAF_LO_21},
                                        {L"ETH_MAF_HI_22 	 ",ETH_MAF_HI_22},
                                        {L"ETH_MAF_LO_22 	 ",ETH_MAF_LO_22},
                                        {L"ETH_MAF_HI_23 	 ",ETH_MAF_HI_23},
                                        {L"ETH_MAF_LO_23 	 ",ETH_MAF_LO_23},
                                        {L"ETH_MAF_HI_24 	 ",ETH_MAF_HI_24},
                                        {L"ETH_MAF_LO_24 	 ",ETH_MAF_LO_24},
                                        {L"ETH_MAF_HI_25 	 ",ETH_MAF_HI_25},
                                        {L"ETH_MAF_LO_25 	 ",ETH_MAF_LO_25},
                                        {L"ETH_MAF_HI_26 	 ",ETH_MAF_HI_26},
                                        {L"ETH_MAF_LO_26 	 ",ETH_MAF_LO_26},
                                        {L"ETH_MAF_HI_27 	 ",ETH_MAF_HI_27},
                                        {L"ETH_MAF_LO_27 	 ",ETH_MAF_LO_27},
                                        {L"ETH_MAF_HI_28 	 ",ETH_MAF_HI_28},
                                        {L"ETH_MAF_LO_28 	 ",ETH_MAF_LO_28},
                                        {L"ETH_MAF_HI_29 	 ",ETH_MAF_HI_29},
                                        {L"ETH_MAF_LO_29 	 ",ETH_MAF_LO_29},
                                        {L"ETH_MAF_HI_30	 ",ETH_MAF_HI_30},
                                        {L"ETH_MAF_LO_30 	 ",ETH_MAF_LO_30},
                                        {L"ETH_MAF_HI_31 	 ",ETH_MAF_HI_31},
                                        {L"ETH_MAF_LO_31 	 ",ETH_MAF_LO_31},
                                        {L"ETH_MAF_HI_32	 ",ETH_MAF_HI_32},
                                        {L"ETH_MAF_LO_32 	 ",ETH_MAF_LO_32},
                                        */
                                        {L"0x500 ETH_WUCSR2 	             ",ETH_WUCSR2},
                           /*           
                                        {L"ETH_WOL_FIFO_STS 	     ",ETH_WOL_FIFO_STS},
                                        {L"ETH_NS1_IPV6_ADDR_DEST0 	 ",ETH_NS1_IPV6_ADDR_DEST0},
                                        {L"ETH_NS1_IPV6_ADDR_DEST1 	 ",ETH_NS1_IPV6_ADDR_DEST1},
                                        {L"ETH_NS1_IPV6_ADDR_DEST2 	 ",ETH_NS1_IPV6_ADDR_DEST2},
                                        {L"ETH_NS1_IPV6_ADDR_DEST3 	 ",ETH_NS1_IPV6_ADDR_DEST3},
                                        {L"ETH_NS1_IPV6_ADDR_SRC0 	 ",ETH_NS1_IPV6_ADDR_SRC0},
                                        {L"ETH_NS1_IPV6_ADDR_SRC1 	 ",ETH_NS1_IPV6_ADDR_SRC1},
                                        {L"ETH_NS1_IPV6_ADDR_SRC2 	 ",ETH_NS1_IPV6_ADDR_SRC2},
                                        {L"ETH_NS1_IPV6_ADDR_SRC3 	 ",ETH_NS1_IPV6_ADDR_SRC3},
                                        {L"ETH_NS1_ICMPV6_ADDR0_0 	 ",ETH_NS1_ICMPV6_ADDR0_0},
                                        {L"ETH_NS1_ICMPV6_ADDR0_1 	 ",ETH_NS1_ICMPV6_ADDR0_1},
                                        {L"ETH_NS1_ICMPV6_ADDR0_2 	 ",ETH_NS1_ICMPV6_ADDR0_2},
                                        {L"ETH_NS1_ICMPV6_ADDR0_3 	 ",ETH_NS1_ICMPV6_ADDR0_3},
                                        {L"ETH_NS1_ICMPV6_ADDR1_0 	 ",ETH_NS1_ICMPV6_ADDR1_0},
                                        {L"ETH_NS1_ICMPV6_ADDR1_1 	 ",ETH_NS1_ICMPV6_ADDR1_1},
                                        {L"ETH_NS1_ICMPV6_ADDR1_2 	 ",ETH_NS1_ICMPV6_ADDR1_2},
                                        {L"ETH_NS1_ICMPV6_ADDR1_3 	 ",ETH_NS1_ICMPV6_ADDR1_3},                        
                                        {L"ETH_NS2_IPV6_ADDR_DEST0 	 ",ETH_NS2_IPV6_ADDR_DEST0},
                                        {L"ETH_NS2_IPV6_ADDR_DEST1 	 ",ETH_NS2_IPV6_ADDR_DEST1},
                                        {L"ETH_NS2_IPV6_ADDR_DEST2 	 ",ETH_NS2_IPV6_ADDR_DEST2},
                                        {L"ETH_NS2_IPV6_ADDR_DEST3 	 ",ETH_NS2_IPV6_ADDR_DEST3},
                                        {L"ETH_NS2_IPV6_ADDR_SRC0 	 ",ETH_NS2_IPV6_ADDR_SRC0},
                                        {L"ETH_NS2_IPV6_ADDR_SRC1 	 ",ETH_NS2_IPV6_ADDR_SRC1},
                                        {L"ETH_NS2_IPV6_ADDR_SRC2 	 ",ETH_NS2_IPV6_ADDR_SRC2},
                                        {L"ETH_NS2_IPV6_ADDR_SRC3 	 ",ETH_NS2_IPV6_ADDR_SRC3},
                                        {L"ETH_NS2_ICMPV6_ADDR0_0 	 ",ETH_NS2_ICMPV6_ADDR0_0},
                                        {L"ETH_NS2_ICMPV6_ADDR0_1 	 ",ETH_NS2_ICMPV6_ADDR0_1},
                                        {L"ETH_NS2_ICMPV6_ADDR0_2 	 ",ETH_NS2_ICMPV6_ADDR0_2},
                                        {L"ETH_NS2_ICMPV6_ADDR0_3 	 ",ETH_NS2_ICMPV6_ADDR0_3},
                                        {L"ETH_NS2_ICMPV6_ADDR1_0 	 ",ETH_NS2_ICMPV6_ADDR1_0},
                                        {L"ETH_NS2_ICMPV6_ADDR1_1 	 ",ETH_NS2_ICMPV6_ADDR1_1},
                                        {L"ETH_NS2_ICMPV6_ADDR1_2 	 ",ETH_NS2_ICMPV6_ADDR1_2},
                                        {L"ETH_NS2_ICMPV6_ADDR1_3 	 ",ETH_NS2_ICMPV6_ADDR1_3},                                       
                                        {L"ETH_SYN_IPV4_ADDR_SRC 	 ",ETH_SYN_IPV4_ADDR_SRC},
                                        {L"ETH_SYN_IPV4_ADDR_DEST 	 ",ETH_SYN_IPV4_ADDR_DEST},
                                        {L"ETH_SYN_IPV4_TCP_PORTS 	 ",ETH_SYN_IPV4_TCP_PORTS},                                     
                                        {L"ETH_SYN_IPV6_ADDR_SRC0 	 ",ETH_SYN_IPV6_ADDR_SRC0},
                                        {L"ETH_SYN_IPV6_ADDR_SRC1 	 ",ETH_SYN_IPV6_ADDR_SRC1},
                                        {L"ETH_SYN_IPV6_ADDR_SRC2 	 ",ETH_SYN_IPV6_ADDR_SRC2},
                                        {L"ETH_SYN_IPV6_ADDR_SRC3 	 ",ETH_SYN_IPV6_ADDR_SRC3},
                                        {L"ETH_SYN_IPV6_ADDR_DEST0 	 ",ETH_SYN_IPV6_ADDR_DEST0},
                                        {L"ETH_SYN_IPV6_ADDR_DEST1 	 ",ETH_SYN_IPV6_ADDR_DEST1},
                                        {L"ETH_SYN_IPV6_ADDR_DEST2 	 ",ETH_SYN_IPV6_ADDR_DEST2},
                                        {L"ETH_SYN_IPV6_ADDR_DEST3 	 ",ETH_SYN_IPV6_ADDR_DEST3},
                                        {L"ETH_SYN_IPV6_TCP_PORTS 	 ",ETH_SYN_IPV6_TCP_PORTS},
                                        */
                                        {L"0x5c0 ETH_ARP_SPA	 ",ETH_ARP_SPA},
                                        {L"0x5c4 ETH_ARP_TPA 	 ",ETH_ARP_TPA}
										};


static const SHOW_REG phyCsr[] ={

							   {L"0x00UL PHY_CTRL         ",PHY_CTRL},
                               {L"0x01UL PHY_STATUS       ",PHY_STATUS},
                               {L"0x02UL PHY_ID1          ",PHY_ID1 },
                               {L"0x03UL PHY_ID2          ",PHY_ID2},
                               {L"0x04UL PHY_AUTONEG_ADV  ",PHY_AUTONEG_ADV},
                               {L"0x05UL PHY_LP_ABILITY   ",PHY_LP_ABILITY },
                               {L"0x06UL PHY_AUTONEG_EXP  ",PHY_AUTONEG_EXP},
                               {L"0x07UL PHY_NEXT_PAGE_TX ",PHY_NEXT_PAGE_TX},
                               {L"0x08UL PHY_LP_NEXT_PAGE ",PHY_LP_NEXT_PAGE},
                               {L"0x09UL PHY_1000T_CTRL   ",PHY_1000T_CTRL },
                               {L"0x0aUL PHY_1000T_STATUS ",PHY_1000T_STATUS},
                               {L"0x0bUL PHY_REG11        ",PHY_REG11 },
                               {L"0x0cUL PHY_REG12        ",PHY_REG12 },
                               {L"0x0dUL PHY_REG13        ",PHY_REG13 },
                               {L"0x0eUL PHY_REG14        ",PHY_REG14 },
                               {L"0x0fUL PHY_EXT_STATUS   ",PHY_EXT_STATUS},
                               {L"0x10UL(16) PHY_LINK_CTL     ",PHY_LINK_CTL },
                               {L"0x11UL(17) PHY_MOD_CTL_STS  ",PHY_MOD_CTL_STS },
                               {L"0x12UL(18) PHY_SPCL_MOD     ",PHY_SPCL_MOD},
                               {L"0x13UL(19) PHY_EXT_CTL_STS  ",PHY_EXT_CTL_STS},
                               {L"0x14UL(20) PHY_SPCL_TST_CNTL ",PHY_SPCL_TST_CNTL },
                               {L"0x15UL(21) PHY_SPCL_TST_READ1",PHY_SPCL_TST_READ1},
                               {L"0x15UL(22) PHY_SPCL_TST_READ2",PHY_SPCL_TST_READ2},
                               {L"0x15UL(23) PHY_SPCL_TST_WRITE",PHY_SPCL_TST_WRITE},
                               {L"0x1bUL(27)(PHY_CTL_STS_IND  ",PHY_CTL_STS_IND},
                               {L"0x1cUL(28) PHY_CH_Q_STS     ",PHY_CH_Q_STS},
                               {L"0x1dUL(29)PHY_INT_SRC_FLG  ",PHY_INT_SRC_FLG},
                               {L"0x1eUL(30)PHY_INT_MASK     ",PHY_INT_MASK },
                               {L"0x1fUL(31) PHY_SPC_CTL_STS  ",PHY_SPC_CTL_STS},
                               };

static const char * LinkSpeed[5]={"UNKNOWN","10Mb","100Mb","1000Mb_Master","1000Mb_Slave"};
static const char * Duplex[3]={"UNKNOWN","HALF","FULL"};



#ifndef min
#define    min(_a, _b)		(((_a) < (_b)) ? (_a) : (_b))
#endif

#ifndef max
#define    max(_a, _b)		(((_a) > (_b)) ? (_a) : (_b))
#endif

//--------------------------------------
// Utility macros		 
//--------------------------------------

#define MP_SET_FLAG(_M, _F) 			((_M)->Flags |= (_F))
#define MP_CLEAR_FLAG(_M, _F)			 ((_M)->Flags &= ~(_F))
#define MP_TEST_FLAG(_M, _F)			(((_M)->Flags & (_F)) != 0)
#define MP_TEST_FLAGS(_M, _F)			 (((_M)->Flags & (_F)) == (_F))

#define MP_IS_READY(_M) 	   (((_M)->Flags & \
								 (fMP_RESET_IN_PROGRESS | fMP_HALT_IN_PROGRESS | fMP_INIT_IN_PROGRESS | fMP_SURPRISE_REMOVED | fMP_DEVICE_NOT_ATTACHED)) == 0) 

#define MP_IS_RESET_PENDING(_M) 	   (((_M)->Flags & \
										 (fMP_NEED_USB_RESET | fMP_NEED_FULL_RESET)) != 0) 

#define MP_IS_READY_AND_LINKED_UP(_M)		 (((_M)->Flags & \
											   (fMP_DISCONNECTED | fMP_RESET_IN_PROGRESS | fMP_HALT_IN_PROGRESS | fMP_INIT_IN_PROGRESS | fMP_SURPRISE_REMOVED | fMP_DEVICE_NOT_ATTACHED)) == 0) 

#define MP_INC_REF(_A)				NdisInterlockedIncrement(&(_A)->RefCount)

#define MP_DEC_REF(_A) {\
							NdisInterlockedDecrement(&(_A)->RefCount);\
							ASSERT(_A->RefCount >= 0);\
							if((_A)->RefCount == 0){\
								NdisSetEvent(&(_A)->HaltEvent);\
							}\
						}

#define MP_GET_REF(_A)				((_A)->RefCount)
 
#include <pshpack1.h>

typedef struct _ETH_HEADER
{
	UCHAR		DstAddr[ETH_LENGTH_OF_ADDRESS];
	UCHAR		SrcAddr[ETH_LENGTH_OF_ADDRESS];
	USHORT		EthType;
} ETH_HEADER, *PETH_HEADER;

#include <poppack.h>

//
// This structure is used to send requests 
//
typedef struct _WDM_NDIS_REQUEST
{
	NDIS_WORK_ITEM		WorkItem; // must be first in the list
	NDIS_OID			Oid;
	NDIS_REQUEST_TYPE	RequestType;
	PVOID				InformationBuffer;
	ULONG				InformationBufferLength;
	PULONG				BytesReadOrWritten;
	PULONG				BytesNeeded;

} WDM_NDIS_REQUEST, *PWDM_NDIS_REQUEST;


// TCB (Transmit Control Block)
typedef struct _TCB
{
	LIST_ENTRY				List; // This must be the first entry
	LONG					Ref;
	PVOID					Adapter;
	PNDIS_PACKET			OrgSendPacket;
	PUCHAR					pData;		// Pointer to start of memory buffer
	ULONG					ulSize;
	BOOLEAN 				bFull;
	ULONG					nPktCount;
	DWORD					dwErr;
	USB_ERROR				dwUsbErr;
	DWORD					dwTransferLength;
	HANDLE					SyncEvent;
} TCB, *PTCB;

// RCB (Receive Control Block)
typedef struct _RCB
{
	LIST_ENTRY				List; // This must be the first entry
	LONG					Ref;
	PVOID					Adapter;
	#ifdef DEBUG
	ULONG					tag;
	#endif
	ULONG					seq;
	PNDIS_BUFFER			Buffer;
	PNDIS_PACKET			Packet;
	PUCHAR					pData;		  
	ULONG					ulSize;
	ULONG					ulCopied;
	USB_ERROR				dwUsbErr;
	DWORD						dwErr;
	DWORD					dwTransferLength;
	HANDLE					SyncEvent;
	ULONG					BulkInRCB;
} RCB, *PRCB;

// ICB (Interrupt Control Block)
typedef struct _ICB
{
	LIST_ENTRY				List; // This must be the first entry
	LONG					Ref;
	PVOID					Adapter;
	PUCHAR					pData;		  
	ULONG					ulSize;
	UCHAR					Data[INT_BUFFER_SIZE];
	USB_ERROR				dwUsbErr;
	DWORD						dwErr;
	DWORD					dwTransferLength;
	HANDLE					SyncEvent;
} ICB, *PICB;

typedef struct _NIC_TASK_OFFLOAD
{
    ULONG   ChecksumOffload;
    ULONG   LargeSendOffload;
    ULONG   IpSecOffload;
}NIC_TASK_OFFLOAD, *PNIC_TASK_OFFLOAD;

// Checksum offload capabilities
typedef struct _NIC_CHECKSUM_OFFLOAD
{
    ULONG   DoXmitTcpChecksum;
    ULONG   DoRcvTcpChecksum;
    ULONG   DoXmitUdpChecksum;
    ULONG   DoRcvUdpChecksum;
    ULONG   DoXmitIpChecksum;
    ULONG   DoRcvIpChecksum;
    
}NIC_CHECKSUM_OFFLOAD, *PNIC_CHECKSUM_OFFLOAD;

// Stats block structures
typedef struct _LAN7500_STATS {
    // Rx Counters
    DWORD   RxCrcErrors;
    DWORD   RxAlignmentErrors;
    DWORD   RxFragmentErrors;
    DWORD   RxJabberErrors;
    DWORD   RxRuntFrameErrors;
    DWORD   RxFrameTooLongErrors;
    DWORD   RxDroppedFrames;
    DWORD   RxGoodUnicastBytes;
    DWORD   RxGoodBroadcastBytes;
    DWORD   RxGoodMulticastBytes;
    DWORD   RxGoodUnicastFrames;
    DWORD   RxGoodBroadcastFrames;
    DWORD   RxGoodMulticastFrames;
    DWORD   RxPauseFrames;
    DWORD   RxGood64ByteFrames;
    DWORD   RxGood65To127ByteFrames;
    DWORD   RxGood128To255ByteFrames;
    DWORD   RxGood256To511ByteFrames;
    DWORD   RxGood512To1023ByteFrames;
    DWORD   RxGood1024To1518ByteFrames;
    DWORD   RxGoodGreater1518ByteFrames;
    
    // Tx Counters
    DWORD   TxCrcErrors;
    DWORD   TxExcessiveDeferralErrors;
    DWORD   TxCarrierErrors;
    DWORD   TxBadBytes;
    DWORD   TxSingleCollisions;
    DWORD   TxMultipleCollisions;
    DWORD   TxExcessiveCollisionErrors;
    DWORD   TxLateCollisionErrors;
    DWORD   TxGoodUnicastBytes;
    DWORD   TxGoodBroadcastBytes;
    DWORD   TxGoodMulticastBytes;
    DWORD   TxGoodUnicastFrames;
    DWORD   TxGoodBroadcastFrames;
    DWORD   TxGoodMulticastFrames;
    DWORD   TxPauseFrames;
    DWORD   TxGood64ByteFrames;
    DWORD   TxGood65To127ByteFrames;
    DWORD   TxGood128To255ByteFrames;
    DWORD   TxGood256To511ByteFrames;
    DWORD   TxGood512To1023ByteFrames;
    DWORD   TxGood1024To1518ByteFrames;
    DWORD   TxGoodGreater1518ByteFrames;
}LAN7500_STATS, *PLAN7500_STATS;
#define NDIS_STATUS_TO_NT_STATUS(_NdisStatus, _pNtStatus)							\
{																					\
																				\
	if (((NDIS_STATUS_SUCCESS == (_NdisStatus)) ||									\
		(NDIS_STATUS_PENDING == (_NdisStatus)) ||									\
		(NDIS_STATUS_BUFFER_OVERFLOW == (_NdisStatus)) ||							\
		(NDIS_STATUS_FAILURE == (_NdisStatus)) ||									\
		(NDIS_STATUS_RESOURCES == (_NdisStatus)) || 								\
		(NDIS_STATUS_NOT_SUPPORTED == (_NdisStatus))))								\
	{																				\
		*(_pNtStatus) = (NTSTATUS)(_NdisStatus);									\
	}																				\
	else if (NDIS_STATUS_BUFFER_TOO_SHORT == (_NdisStatus)) 						\
	{																				\
																				 \
		*(_pNtStatus) = STATUS_BUFFER_TOO_SMALL;									\
	}																				\
	else if (NDIS_STATUS_INVALID_LENGTH == (_NdisStatus))							\
	{																				\
		*(_pNtStatus) = STATUS_INVALID_BUFFER_SIZE; 								\
	}																				\
	else if (NDIS_STATUS_INVALID_DATA == (_NdisStatus)) 							\
	{																				\
		*(_pNtStatus) = STATUS_INVALID_PARAMETER;									\
	}																				\
	else if (NDIS_STATUS_ADAPTER_NOT_FOUND == (_NdisStatus))						\
	{																				\
		*(_pNtStatus) = STATUS_NO_MORE_ENTRIES; 									\
	}																				\
	else if (NDIS_STATUS_ADAPTER_NOT_READY == (_NdisStatus))						\
	{																				\
		*(_pNtStatus) = STATUS_DEVICE_NOT_READY;									\
	}																				\
	else																			\
	{																				\
		*(_pNtStatus) = STATUS_UNSUCCESSFUL;										\
	}																				\
}

#define NT_STATUS_TO_NDIS_STATUS(_NtStatus, _pNdisStatus)							\
{																					\
																			   \
	if (((STATUS_SUCCESS == (_NtStatus)) || 								 \
		(STATUS_PENDING == (_NtStatus)) ||									 \
		(STATUS_BUFFER_OVERFLOW == (_NtStatus)) ||							 \
		(STATUS_UNSUCCESSFUL == (_NtStatus)) || 								  \
		(STATUS_INSUFFICIENT_RESOURCES == (_NtStatus)) ||								  \
		(STATUS_NOT_SUPPORTED == (_NtStatus)))) 							 \
	{																				\
		*(_pNdisStatus) = (NDIS_STATUS)(_NtStatus); 								   \
	}																				\
	else if (STATUS_BUFFER_TOO_SMALL == (_NtStatus))						 \
	{																				\
																				 \
		*(_pNdisStatus) = NDIS_STATUS_BUFFER_TOO_SHORT; 						   \
	}																				\
	else if (STATUS_INVALID_BUFFER_SIZE == (_NtStatus)) 						  \
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_INVALID_LENGTH;								  \
	}																				\
	else if (STATUS_INVALID_PARAMETER == (_NtStatus))							  \
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_INVALID_DATA; 								  \
	}																				\
	else if (STATUS_NO_MORE_ENTRIES == (_NtStatus)) 					   \
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_ADAPTER_NOT_FOUND;									 \
	}																				\
	else if (STATUS_DEVICE_NOT_READY == (_NtStatus))						\
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_ADAPTER_NOT_READY;									\
	}																				\
	else																			\
	{																				\
		*(_pNdisStatus) = NDIS_STATUS_FAILURE;										  \
	}																				\
}



#endif	  // _LAN7500_H



