/*****************************************************************************
	Copyright (c) 2009-2010 SMSC. All rights reserved.

	The information contained in this file is confidential and proprietary to
	Standard Microsystems Corporation.  No part of this file may be reproduced
	or distributed, in any form or by any means for any purpose, without the
	express written permission of Standard Microsystems Corporation.
	
	 
	File name   : os.h
	Description : OS releted Header file

	History	   : 01/19/2010    Initial version.		Nancy Lin
		
*****************************************************************************/
#ifndef _OS_H_
#define _OS_H_

#ifdef	_lint
#define	__cdecl	
#endif

// some lint error suppressions for WinCE headers
// -e14		// Symbol previously defined
// -e43		// Vacuous type for variable
// -e46		// field type should be int
// -e123	// Macro defined with arguments someplace else
// -e427	// "//" comment terminates in "\"
// -e537	// Repeated include file
// -e620	// Suspicious constant (L or one?)
// -e652	// #define of symbol declared previously
// -e659	// Nothing follows '}' on line within struct/union/enum declaration
// -e683	// function #define'd, semantics may be lost

// -e726	// Extraneous comma ignored
// -e760	// Redundant macro xxx defined identically at line yyy
// -e761	// Redundant typedef previously declared
// -e762	// Redundantly declared symbol
// -e763	// Redundant declaration for symbol
// -e767	// macro was defined differently in another module 
// -e773	// Expression-like macro 'va_start' not parenthesized
// -e793	// ANSI limit of 8 'conditional inclusion levels' exceeded -- processing is unaffected
// -e806	// Small bit field is signed rather than unsigned
// -e828	// Semantics of function 'setjmp' copied to function '_setjmp'

// -e935	// int within struct
// -e937	// old-style function declaration for function
// -e950	// Non-ANSI reserved word or construct
// -e955	// Parameter name missing from prototype for function
// -e956	// Non const, non volatile static or external variable
// -e957	// Function defined without a prototype in scope
// -e958	// Padding is required to align member on 'n' byte boundary
// -e959	// Nominal struct size is not an even multiple of the maximum member alignment
// -e960	// a whole bunch of invidiaul MISRA Required Rules
// -e961	// a whole bunch of individual MISRA Advisory Rules
// -e962	// Macro defined identically at another location 
// -e973	// Unary operator in macro not parenthesized
// -e1916	// Ellipsis encountered
// -e912	// Implicit binary conversion from int to unsigned int

/*lint -save*/
/*lint -e14 -e43 -e46 -e123 -e427 -e537 -e620 -e652 -e659 -e683*/
/*lint -e726 -e760 -e761 -e762 -e763 -e767 -e773 -e793 -e806 -e828 -e912*/
/*lint -e935 -e937 -e950 -e955 -e956 -e957 -e958 -e959 -e960 -e961 -e962*/
/*lint -e973 -e1916*/
#include <ndis.h>
#include <winreg.h>
#include <nkintr.h>
#include <Pkfuncs.h>
#include <pm.h>
 

//
//  DEBUG MACROS
//
#define DBG_INIT			0x00000001UL
#define DBG_TX				0x00000002UL
#define DBG_RX				0x00000004UL
#define DBG_PHY				0x00000008UL
#define DBG_SHUT_DOWN		0x00000010UL
#define DBG_INT        		0x00000020UL
#define DBG_RESET			0x00000040UL
#define DBG_MULTICAST  		0x00000080UL
#define DBG_FLOW            0x00000100UL
#define DBG_POWER           0x00000200UL
#define DBG_EEPROM          0x00000400UL
#define DBG_LINK          	0x00000800UL
#define DBG_USB_INIT        0x00001000UL
#define DBG_USB_RESET       0x00002000UL
#define USB_RECOVERY	    0x00004000UL
#define DBG_REQUEST     	0x00008000UL
#define DBG_E2PRAM        0x00010000UL
#define DBG_ERROR    		0x20000000UL
#define DBG_WARNING 		0x10000000UL

#define SMSC_DBG

#ifdef SMSC_DBG
extern DWORD g_DebugMode;

#define SMSC_SET_DEBUG_MODE(debugMode) {g_DebugMode=debugMode;}


#define	SMSC_WARNING(printf_exp)						\
		{												\
			RETAILMSG(1, (TEXT("SMSC_WARNING: ")));		\
			RETAILMSG(1, (TEXT(" Func:%s, Line:%d "), __FILE__, __LINE__));	\
			RETAILMSG(1, (printf_exp));					\
		}

#define SMSC_WARNING0(message)							\
		{						                        \
            RETAILMSG(1, (TEXT("SMSC_WARNING: ")));     \
            RETAILMSG(1, (TEXT(message)));              \
		}

#define SMSC_WARNING1(message, arg1)					    \
		{                       						\
            RETAILMSG(1, (TEXT("SMSC_WARNING: ")));     \
            RETAILMSG(1, (TEXT(message), arg1));         \
		}

#define SMSC_WARNING2(message, arg1, arg2)				\
		{                       						\
            RETAILMSG(1, (TEXT("SMSC_WARNING: ")));     \
            RETAILMSG(1, (TEXT(message), arg1, arg2));  \
		}

#define SMSC_WARNING3(message, arg1, arg2, arg3)		        \
		{                           						    \
            RETAILMSG(1, (TEXT("SMSC_WARNING: ")));             \
            RETAILMSG(1, (TEXT(message), arg1, arg2, arg3));    \
		}

#define SMSC_TRACE0(job, message)							\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message)));                  \
		}

#define SMSC_TRACE1(job, message, arg1)						\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message), arg1));             \
		}

#define SMSC_TRACE2(job, message, arg1, arg2)				\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message), arg1, arg2));      \
		}

#define SMSC_TRACE3(job, message, arg1, arg2, arg3)			\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message), arg1, arg2, arg3));    \
		}

#define SMSC_TRACE4(job, message, arg1, arg2, arg3, arg4)	\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message), arg1, arg2, arg3, arg4));    \
		}

#define SMSC_TRACE5(job, message, arg1, arg2, arg3, arg4, arg5)	\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message), arg1, arg2, arg3, arg4, arg5));    \
		}

#define SMSC_TRACE6(job, message, arg1, arg2, arg3, arg4, arg5, arg6)	\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message), arg1, arg2, arg3, arg4, arg5, arg6)); \
		}

#define SMSC_TRACE7(job, message, arg1, arg2, arg3, arg4, arg5, arg6,arg7)	\
		if (job&g_DebugMode) {					\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT(message), arg1, arg2, arg3, arg4, arg5, arg6,arg7)); \
		}

#define SMSC_ASSERT(condition)		\
		if(!(condition)) {			\
			RETAILMSG(1, (TEXT("SMSC: ")));     \
            RETAILMSG(1, (TEXT("SMSC_ASSERTION_FAILURE: File=%s, Line=%d\r\n"), TEXT(__FILE__), __LINE__));    \
			SMSC_BREAK;				\
		}

#else	// SMSC_DBG
#define SMSC_SET_DEBUG_MODE(debugMode) 
#define SMSC_WARNING(printf_exp)
#define SMSC_WARNING0(msg)
#define SMSC_WARNING1(msg,arg1)
#define SMSC_WARNING2(msg,arg1,arg2)
#define SMSC_WARNING3(msg,arg1,arg2,arg3)
#define SMSC_ASSERT(condition)
#define SMSC_TRACE0(job, msg)
#define SMSC_TRACE1(job, msg,arg1)
#define SMSC_TRACE2(job, msg,arg1,arg2)
#define SMSC_TRACE3(job, msg,arg1,arg2,arg3)
#define SMSC_TRACE4(job, msg,arg1,arg2,arg3,arg4)
#define SMSC_TRACE5(job, msg,arg1,arg2,arg3,arg4,arg5)
#define SMSC_TRACE6(job, msg,arg1,arg2,arg3,arg4,arg5,arg6)
#define SMSC_TRACE7(job, msg,arg1,arg2,arg3,arg4,arg5,arg6,arg7)
#endif	// SMSC_DBG
#endif // _OS_H_
